@@ -23,22 +23,21 @@
 model abstraction for administration clients supporting different views
 """
 
-import os
 import sys
-import md5
 
 import gobject
 
 from twisted.spread import pb
 from twisted.internet import error, defer, reactor
 from twisted.cred import error as crederror
-from twisted.python import rebuild, reflect
+from twisted.python import rebuild, reflect, failure
 
 from flumotion.common import common, errors, interfaces, log, pygobject
-from flumotion.common import keycards, worker, planet, medium, package
+from flumotion.common import keycards, worker, planet, medium, package, messages
 # serializable worker and component state
 from flumotion.twisted import flavors
 from flumotion.twisted.defer import defer_generator_method
+from flumotion.twisted.compat import implements
 
 from flumotion.configure import configure
 from flumotion.common import reload
@@ -47,6 +46,9 @@ from flumotion.twisted import pb as fpb
 
 from flumotion.common.pygobject import gsignal, gproperty
 
+from flumotion.common.messages import N_
+T_ = messages.gettexter('flumotion')
+
 # FIXME: keycard should be created and handed to factory instead
 class AdminClientFactory(fpb.ReconnectingFPBClientFactory):
     def __init__(self, medium, user, passwd):
@@ -138,7 +140,7 @@ class AdminClientFactory(fpb.ReconnectingFPBClientFactory):
             self.debug("emitted connection-refused")
 
         except Exception, e:
-            self.medium._defaultErrback(e)
+            self.medium._defaultErrback(failure.Failure(e))
 
     gotDeferredLogin = defer_generator_method(gotDeferredLogin)
         
@@ -164,7 +166,7 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
 
     logCategory = 'adminmodel'
 
-    __implements__ = interfaces.IAdminMedium, flavors.IStateListener,
+    implements(interfaces.IAdminMedium, flavors.IStateListener)
 
     # Public instance variables (read-only)
     planet = None
@@ -280,6 +282,9 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
                          '<passwd>%s</passwd>' % self.passwd,
                          '</connection>'])
             
+            # defgen + pychecker quirkiness
+            import os
+            import md5
             sum = md5.new(s).hexdigest()
             f = os.path.join(configure.registrydir, '%s.connection' % sum)
             h = open(f, 'w')
@@ -333,6 +338,9 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
         self.log('remote: %s: %s: %s' % (type, category, message))
         
     def remote_componentCall(self, componentState, methodName, *args, **kwargs):
+        """
+        @todo 0.4: rename this method to remote_componentViewCall
+        """
         self.callViews('componentCall',
             componentState, methodName, *args, **kwargs)
 
@@ -397,8 +405,25 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
                             componentState, methodName,
                             *args, **kwargs)
         d.addCallback(self._callRemoteCallback, methodName, componentName)
-        d.addErrback(self._callRemoteErrback, "component",
-                     componentName, methodName)
+        def errback(failure):
+            msg = None
+            if failure.check(errors.NoMethodError):
+                msg = "Remote method '%s' does not exist." % methodName
+                msg += "\n" + failure.value
+            else:
+                msg = failure.value
+
+            # FIXME: we probably need a nicer way of getting component
+            # messages shown from the admin model, but this allows us to
+            # make sure every type of admin has these messages
+            self.warning(msg)
+            m = messages.Warning(T_(N_("Internal error in component.")),
+                debug=msg)
+            componentState.observe_append('messages', m)
+            return failure
+
+        d.addErrback(errback)
+        # FIXME: dialog for other errors ?
         return d
 
     def _callRemoteCallback(self, result, methodName, componentName):
@@ -427,8 +452,8 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
 
     def _callRemoteErrback(self, failure, type, name, methodName):
         if failure.check(errors.NoMethodError):
-            self.warning("method %s on %s does not exist, component bug" % (
-                methodName, name))
+            self.warning("method '%s' on component '%s' does not exist, "
+                "component bug" % (methodName, name))
         else:
             self.debug("passing through failure on remote call to %s(%s): %r" %
                 (name, methodName, failure))
@@ -452,14 +477,19 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
                                         element, property)
 
     ## reload methods for everything
-    def reload(self):
-        # XXX: reload admin.py too
+    def reloadAdmin(self):
         name = reflect.filenameToModuleName(__file__)
 
-        self.info("rebuilding '%s'" % name)
+        self.info('Reloading admin code')
+        self.debug("rebuilding '%s'" % name)
         rebuild.rebuild(sys.modules[name])
+        self.debug("reloading '%s'" % name)
+        reload.reload()
+        self.info('Reloaded admin code')
 
-        d = defer.execute(reload.reload)
+    def reload(self):
+        # XXX: reload admin.py too
+        d = defer.execute(self.reloadAdmin)
 
         d = d.addCallback(lambda result, self: self.reloadManager(), self)
         d.addErrback(self._defaultErrback)
@@ -470,6 +500,8 @@ class AdminModel(medium.BaseMedium, gobject.GObject):
             d.addErrback(self._defaultErrback)
         return d
 
+    # used by other admin clients
+    # FIXME: isn't it great how hard it is to guess what duckport is ?
     def reload_async(self, duckport):
         name = reflect.filenameToModuleName(__file__)
 
