@@ -41,18 +41,23 @@ from flumotion.configure import configure
 from flumotion.common import errors, log, worker, planet, common, pygobject
 from flumotion.manager import admin # Register types
 from flumotion.twisted import flavors, reflect
+from flumotion.twisted.compat import implements
 from flumotion.ui import icons, trayicon
 
 from flumotion.common.planet import moods
 from flumotion.common.pygobject import gsignal
 
+from flumotion.common import messages
+from flumotion.common.messages import N_
+T_ = messages.gettexter('flumotion')
+
 class Window(log.Loggable, gobject.GObject):
     '''
     Creates the GtkWindow for the user interface.
     Also connects to the manager on the given host and port.
     '''
 
-    __implements__ = flavors.IStateListener,
+    implements(flavors.IStateListener)
     
     logCategory = 'adminview'
     gsignal('connected')
@@ -60,17 +65,19 @@ class Window(log.Loggable, gobject.GObject):
     def __init__(self, model):
         self.__gobject_init__()
         
-        self.current_component_state = None
-
         self.widgets = {}
         self.debug('creating UI')
         self._trayicon = None
 
+        # current component's UI;
+        # L{flumotion.component.base.admin_gtk.BaseAdminGtk}
+        self.current_component = None
+        self.current_component_state = None # its state
+
         self._create_ui()
 
         self._append_recent_connections()
 
-        self.current_component = None # the component we're showing UI for
         self._disconnected_dialog = None # set to a dialog if we're
                                          # disconnected
 
@@ -153,7 +160,8 @@ class Window(log.Loggable, gobject.GObject):
         self._trayicon = trayicon.FluTrayIcon(self)
         self._trayicon.set_tooltip(_('Not connected'))
 
-        self.hpaned = widgets['hpaned']
+        # the widget containing the component view
+        self._sidepane = widgets['hpaned']
  
         window.connect('delete-event', self.close)
 
@@ -249,10 +257,13 @@ class Window(log.Loggable, gobject.GObject):
         @param entryPath:  absolute path to the cached base directory
         @param fileName:   path to the file with the entry point, under
                            entryPath
-        @param methodName: name of the method to instantiate the UI view
+        @param methodName: name of the method to instantiate the
+                           L{flumotion.component.base.admin_gtk.BaseAdminGtk}
+                           UI view
         @param data:       the python code to load
         """
         # methodName has historically been GUIClass
+
         instance = None
 
         name = state.get('name')
@@ -289,9 +300,18 @@ class Window(log.Loggable, gobject.GObject):
 
         # check if we have the method
         if not hasattr(module, methodName):
-            self.warning('method %s not found in file %s' % (
-                methodName, fileName))
-            raise #FIXME: something appropriate
+            msg = 'method %s not found in file %s' % (
+                methodName, fileName)
+            self.warning(msg)
+
+            m = messages.Error(T_(
+                N_("This component has a UI bug.")),
+                    debug=msg,
+                    id=methodName)
+            self._messages_view.add_message(m)
+
+            # FIXME: something more detailed as an error ?
+            raise errors.FlumotionError(msg)
         klass = getattr(module, methodName)
 
         # instantiate the GUIClass
@@ -328,29 +348,36 @@ class Window(log.Loggable, gobject.GObject):
             notebook.append_page(table, gtk.Label(node.title))
             
         # put "loading" widget in
-        old = self.hpaned.get_child2()
-        self.hpaned.remove(old)
-        self.hpaned.add2(notebook)
+        old = self._sidepane.get_child2()
+        self._sidepane.remove(old)
+        self._sidepane.add2(notebook)
         notebook.show_all()
 
         # trigger node rendering
-        # FIXME: might be better to do these one by one, in order,
-        # so the status bar can show what happens
+        d = defer.Deferred()
+
         for node in nodes.values():
             mid = self.statusbar.push('notebook',
                 _("Loading tab %s for %s ...") % (node.title, name))
             node.statusbar = self.statusbar # hack
-            d = node.render()
+            self.debug('adding callback for %s node.render()' % node.title)
+            d.addCallback(lambda _, n: n.render(), node)
             d.addCallback(self._nodeRenderCallback, node.title,
-                instance, nodeWidgets, mid)
+                nodeWidgets, mid)
             d.addErrback(self._nodeRenderErrback, node.title)
 
-    def _nodeRenderCallback(self, widget, nodeName, gtkAdminInstance,
-        nodeWidgets, mid):
+        d.addCallback(self._setCurrentComponentCallback, instance)
+
+        d.callback(None)
+        return d
+
+    # called when one node gets rendered
+    def _nodeRenderCallback(self, widget, nodeName, nodeWidgets, mid):
         # used by show_component
         self.debug("Got sub widget %r" % widget)
         self.statusbar.remove('notebook', mid)
 
+        # clear out any old node widgets with the same name
         table = nodeWidgets[nodeName]
         for w in table.get_children():
             table.remove(w)
@@ -366,13 +393,16 @@ class Window(log.Loggable, gobject.GObject):
         table.add(widget)
         widget.show()
 
-        self.current_component = gtkAdminInstance
-
     def _nodeRenderErrback(self, failure, nodeName):
-        self.debug('Could not render node %s: %r, %s' % (nodeName,
-            failure, failure.getErrorMessage()))
+        self.debug('Could not render node %s: %s' % (nodeName,
+            log.getFailureMessage(failure)))
         self.warning('Could not render node %s' % nodeName)
 
+    def _setCurrentComponentCallback(self, _, instance):
+        self.debug('setting current_component to %r' % instance)
+        self.current_component = instance
+
+
     ### IAdminView interface methods: FIXME: create interface somewhere
     ## Confusingly enough, this procedure is called by remote objects to
     ## operate on the client ui. I think. It is *not* for calling
@@ -492,6 +522,7 @@ class Window(log.Loggable, gobject.GObject):
             if value == moods.sleeping.value:
                 if state.get('name') == current:
                     self._messages_view.clear()
+                    self._sidepane_clear()
 
     def stateAppend(self, state, key, value):
         if isinstance(state, worker.AdminWorkerHeavenState):
@@ -533,21 +564,15 @@ class Window(log.Loggable, gobject.GObject):
             if key == 'names':
                 self.statusbar.set('main', 'Worker %s logged out.' % value)
         elif isinstance(state, planet.AdminFlowState):
+            # the basic admin client only looks at the "default" flow
             if state.get('name') != 'default':
                 return
+
             if key == 'components':
-                name = value.get('name')
-                self.debug('removing component %s' % name)
-                del self._components[name]
-                # FIXME: would be nicer to do this incrementally instead
-                self.update_components()
+                self._remove_component(value)
         elif isinstance(state, planet.AdminAtmosphereState):
             if key == 'components':
-                name = value.get('name')
-                self.debug('removing component %s' % name)
-                del self._components[name]
-                # FIXME: would be nicer to do this incrementally instead
-                self.update_components()
+                self._remove_component(value)
         elif isinstance(state, planet.AdminPlanetState):
             self.debug('something got removed from the planet')
             pass
@@ -558,9 +583,27 @@ class Window(log.Loggable, gobject.GObject):
                 current = self.components_view.get_selected_name()
                 if name == current:
                     self._messages_view.clear_message(value.id)
+            self._set_stop_start_component_sensitive()
         else:
             self.warning('stateRemove of key %s and value %r on unknown object %r' % (key, value, state))
 
+    def _remove_component(self, state):
+        name = state.get('name')
+        self.debug('removing component %s' % name)
+        del self._components[name]
+
+        # if this component was selected, clear selection
+        if self.current_component_state == state:
+            self.debug('removing currently selected component state')
+            self.current_component = None
+            self.current_component_state = None
+        # FIXME: would be nicer to do this incrementally instead
+        self.update_components()
+
+        # a component being removed means our selected component could
+        # have gone away
+        self._set_stop_start_component_sensitive()
+
     ### admin model callbacks
     def admin_connected_cb(self, admin):
         self.info('Connected to manager')
@@ -673,6 +716,16 @@ class Window(log.Loggable, gobject.GObject):
         can_stop = bool(moodname and moodname!='sleeping' and moodname!='lost')
         d['menuitem_manage_stop_component'].set_sensitive(can_stop)
         d['toolbutton_stop_component'].set_sensitive(can_stop)
+        self.debug('can start %r, can stop %r' % (can_start, can_stop))
+
+    # clear the component view in the sidepane.  Called when the current
+    # component goes sleeping
+    def _sidepane_clear(self):
+        old = self._sidepane.get_child2()
+        self._sidepane.remove(old)
+        sub = gtk.Label("")
+        self._sidepane.add2(sub)
+        sub.show()
 
     ### ui callbacks
     def _components_view_has_selection_cb(self, view, state):
@@ -693,11 +746,11 @@ class Window(log.Loggable, gobject.GObject):
         messages = state.get('messages')
         self._messages_view.clear()
         # no ui, clear; FIXME: do this nicer
-        old = self.hpaned.get_child2()
-        self.hpaned.remove(old)
+        old = self._sidepane.get_child2()
+        self._sidepane.remove(old)
         #sub = gtk.Label('%s does not have a UI yet' % name)
         sub = gtk.Label("")
-        self.hpaned.add2(sub)
+        self._sidepane.add2(sub)
         sub.show()
 
         if messages:
@@ -981,6 +1034,9 @@ class Window(log.Loggable, gobject.GObject):
     def debug_reload_manager_cb(self, button):
         self.admin.reloadManager()
 
+    def debug_reload_admin_cb(self, button):
+        self.admin.reloadAdmin()
+
     def debug_reload_all_cb(self, button):
         # FIXME: move all of the reloads over to this dialog
         def _stop(dialog):
