@@ -30,6 +30,7 @@ import gtk.glade
 from flumotion.configure import configure
 from flumotion.common import log, planet, pygobject
 from flumotion.twisted import flavors
+from flumotion.twisted.compat import implements
 
 from flumotion.common.planet import moods
 from flumotion.common.pygobject import gsignal, gproperty
@@ -130,7 +131,7 @@ class ComponentsView(log.Loggable, gobject.GObject):
     I present a view on the list of components logged in to the manager.
     """
     
-    __implements__ = flavors.IStateListener,
+    implements(flavors.IStateListener)
 
     logCategory = 'components'
     
@@ -391,6 +392,8 @@ class ComponentsView(log.Loggable, gobject.GObject):
                 self._model.set(iter, COL_NAME, value)
         elif key == 'workerName':
             self._updateWorker(iter, state)
+        elif key == 'cpu':
+            self._model.set(iter, COL_CPU, value)
 
     # FIXME: proxy messages to message area
     def stateAppend(self, state, key, value):
