@@ -32,6 +32,7 @@ import signal
 
 from twisted.python import reflect, rebuild, components
 from twisted.internet import address
+import twisted.copyright
 
 from flumotion.common import log
 
@@ -134,13 +135,27 @@ def mergeImplements(*classes):
     """
     Merge the __implements__ tuples of the given classes into one tuple.
     """
-    allYourBase = []
-    for clazz in classes:
-        allYourBase += getattr(clazz, '__implements__', ())
-    return tuple(allYourBase)
+    if twisted.copyright.version[0] < '2':
+        allYourBase = []
+        for clazz in classes:
+            allYourBase += getattr(clazz, '__implements__', ())
+        return tuple(allYourBase)
+    else:
+        allYourBase = ()
+        for clazz in classes:
+            try:
+                interfaces = [i for i in clazz.__implemented__]
+            except AttributeError:
+                # with twisted 2.0.1, we get AttributeError with a simple
+                # class C: pass
+                # which does not have C.__implemented__
+                interfaces = []
+            for interface in interfaces:
+                allYourBase += (interface,)
+        return allYourBase
 
-
-def daemonize(stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):
+def daemonize(stdin='/dev/null', stdout='/dev/null', stderr='/dev/null',
+              directory='/'):
     '''
     This forks the current process into a daemon.
     The stdin, stdout, and stderr arguments are file names that
@@ -150,6 +165,8 @@ def daemonize(stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):
     Note that stderr is opened unbuffered, so
     if it shares a file with stdout then interleaved output
     may not appear in the order that you expect.
+
+    The fork will switch to the given directory.
     '''
 
     # first fork
@@ -162,7 +179,12 @@ def daemonize(stdin='/dev/null', stdout='/dev/null', stderr='/dev/null'):
         sys.exit(1)
 
     # decouple from parent environment
-    os.chdir("/") 
+    try:
+        os.chdir(directory) 
+    except OSError, e: 
+        from flumotion.common import errors
+        raise errors.SystemError, "Failed to change directory to %s: %s" % (
+            directory, e.strerror)
     os.umask(0) 
     os.setsid() 
 
