@@ -398,7 +398,10 @@ class FlumotionConfigXML(log.Loggable):
     def _get_string_value(self, nodes):
         values = []
         for subnode in nodes:
-            data = subnode.childNodes[0].data
+            try:
+                data = subnode.childNodes[0].data
+            except IndexError:
+                continue
             # libxml always gives us unicode, even when we encode values
             # as strings. try to make normal strings again, unless that
             # isn't possible.
@@ -416,8 +419,11 @@ class FlumotionConfigXML(log.Loggable):
     def _get_raw_string_value(self, nodes):
         values = []
         for subnode in nodes:
-            data = str(subnode.childNodes[0].data)
-            values.append(data)
+            try:
+                data = str(subnode.childNodes[0].data)
+                values.append(data)
+            except IndexError: # happens on a subnode without childNOdes
+                pass
 
         string = "".join(values)
         return [string, ]
@@ -466,8 +472,8 @@ class FlumotionConfigXML(log.Loggable):
         # at this point we don't support assigning certain sources to
         # certain eaters -- a problem to fix later. for now take the
         # union of the properties.
-        required = True in [x.getRequired() for x in eaters.values()]
-        multiple = True in [x.getMultiple() for x in eaters.values()]
+        required = [x for x in eaters.values() if x.getRequired()]
+        multiple = [x for x in eaters.values() if x.getMultiple()]
 
         if len(strings) == 0 and required:
             raise ConfigError("Component %s wants to eat on %s, but no "
