@@ -56,7 +56,26 @@ class PipelineParseError(pb.Error):
 
 # remote method errors
 class RemoteMethodError(pb.Error):
-    "Generic remote method error"
+    """
+    Generic remote method error.
+
+    @ivar methodName: name of the method
+    @type methodName: str
+    @ivar debug:      optional additional debug message
+    @type debug:      str
+    """
+    def __init__(self, methodName, debug=None):
+        self.methodName = methodName
+        self.debug = debug
+        # work like a normal Exception too
+        self.args = (methodName, debug)
+
+    # this allows us to decide how it gets serialized
+    def __str__(self):
+        msg = "%s on method '%s'" % (self.__class__.__name__, self.methodName)
+        if self.debug:
+            msg += " (%s)" % self.debug
+        return msg
 
 class RemoteRunError(RemoteMethodError):
     "Error while running remote code, before getting a result"
@@ -126,6 +145,13 @@ class ComponentStartError(ComponentError):
     remote_start call on a component.
     """
 
+class ComponentStartHandledError(ComponentStartError):
+    """
+    An error during starting of a component, that's already handled in a
+    different way (for example, through a message).
+    Can be raised during a remote_start call on a component.
+    """
+
 class UnknownComponentError(ComponentError):
     "A given component or component type does not exist"
 
