@@ -115,20 +115,26 @@ def get_plugin_version(plugin_name):
     """
     Find the version of the given plugin.
 
-    @rtype: tuple of (major, minor, micro, [nano]), or None if it could not be
-            found or determined
+    @return: tuple of (major, minor, micro [, nano]),
+             or None if it could not be found or determined
+    @rtype:  tuple of int or None
     """
     __pychecker__ = 'no-moduleattr'
 
-    plugin = None
-
+    pluginVersion = None
     if gst.pygst_version >= (0, 9, 0):
         plugin = gst.registry_get_default().find_plugin(plugin_name)
+        if plugin:
+            l = [int(x) for x in plugin.get_version().split('.')]
+            pluginVersion = tuple(l)
     elif gst.pygst_version > (0, 8, 1):
         # API added after gst-python 0.8.1 release
         plugin = gst.registry_pool_find_plugin(plugin_name)
+        pluginVersion = plugin.get_version()
 
-    if not plugin:
+    if not pluginVersion:
         return None
     else:
-        return plugin.get_version()
+        if len(pluginVersion) < 4:
+            pluginVersion = pluginVersion + (0,)
+        return pluginVersion
