@@ -2,7 +2,8 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005 Fluendo, S.L. (www.fluendo.com). All rights reserved.
+# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
 # the GNU General Public License version 2 as published by
@@ -29,9 +30,7 @@ import errno
 from xml.dom import minidom, Node
 from xml.parsers import expat
 
-from twisted.python import reflect
-
-from flumotion.common import common, log
+from flumotion.common import common, log, package, bundle, errors
 from flumotion.configure import configure
 
 __all__ = ['ComponentRegistry', 'registry']
@@ -46,9 +45,21 @@ def _getMTime(file):
     return os.stat(file)[stat.ST_MTIME]
 
 class RegistryEntryComponent:
-    "This class represents a <component> entry in the registry"
+    """
+    I represent a <component> entry in the registry
+    """
+    # RegistryEntryComponent has a constructor with a lot of arguments,
+    # but that's ok here. Allow it through pychecker.
+    __pychecker__ = 'maxargs=12'
+
     def __init__(self, filename, type, 
-                 source='', base='', properties=[], files=[], entries=[]):
+                 source, base, properties, files,
+                 entries, eaters, feeders, needs_sync, clock_priority):
+        """
+        @type properties:  dict of str -> L{RegistryEntryProperty}
+        @type entries:     dict of str -> L{RegistryEntryEntry}
+        @param entries:    dict of type -> entry
+        """
         self.filename = filename
         self.type = type
         self.source = source
@@ -56,9 +67,24 @@ class RegistryEntryComponent:
         self.properties = properties
         self.files = files
         self.entries = entries
+        self.eaters = eaters
+        self.feeders = feeders
+        self.needs_sync = needs_sync
+        self.clock_priority = clock_priority
         
     def getProperties(self):
-        return self.properties
+        """
+        Get a list of all properties.
+
+        @rtype: list of L{RegistryEntryProperty}
+        """
+        return self.properties.values()
+
+    def hasProperty(self, name):
+        """
+        Check if the component has a property with the given name.
+        """
+        return name in self.properties.keys()
 
     def getFiles(self):
         return self.files
@@ -67,6 +93,11 @@ class RegistryEntryComponent:
         return self.entries.values()
 
     def getEntryByType(self, type):
+        """
+        Get the entry point for the given type of entry.
+
+        @type type: string
+        """
         return self.entries[type]
     
     def getGUIEntry(self):
@@ -88,10 +119,24 @@ class RegistryEntryComponent:
     def getSource(self):
         return self.source
     
+    def getEaters(self):
+        return self.eaters
+
+    def getFeeders(self):
+        return self.feeders
+
+    def getNeedsSynchronization(self):
+        return self.needs_sync
+
+    def getClockPriority(self):
+        return self.clock_priority
+
 class RegistryEntryBundle:
     "This class represents a <bundle> entry in the registry"
-    def __init__(self, name, dependencies, directories):
+    def __init__(self, name, project, under, dependencies, directories):
         self.name = name
+        self.project = project
+        self.under = under
         self.dependencies = dependencies
         self.directories = directories
 
@@ -107,6 +152,16 @@ class RegistryEntryBundle:
     def getDirectories(self):
         return self.directories
     
+    def getProject(self):
+        return self.project
+
+    def getBaseDir(self):
+        if self.project == 'flumotion':
+            return getattr(configure, self.under)
+
+        from flumotion.project import project
+        return project.get(self.project, self.under)
+    
 class RegistryEntryBundleDirectory:
     "This class represents a <directory> entry in the registry"
     def __init__(self, name, files):
@@ -185,9 +240,32 @@ class RegistryEntryEntry:
     def getLocation(self):
         return self.location
 
+    def getModuleName(self, base=None):
+        if base:
+            path = os.path.join(base, self.getLocation())
+        else:
+            path = self.getLocation()
+        return common.pathToModuleName(path)
+
     def getFunction(self):
         return self.function
     
+class RegistryEntryEater:
+    "This class represents a <eater> entry in the registry"
+    def __init__(self, name, required=True, multiple=False):
+        self.name = name
+        self.required = required
+        self.multiple = multiple
+
+    def getName(self):
+        return self.name
+
+    def getRequired(self):
+        return self.required
+
+    def getMultiple(self):
+        return self.multiple
+    
 class XmlParserError(Exception):
     "Error during parsing of XML."
 
@@ -226,7 +304,7 @@ class RegistryParser(log.Loggable):
             self.debug('Parsing XML string')
             return minidom.parseString(string)
         else:
-            self.debug('Parsing XML file: %s' % os.path.basename(filename))
+            self.debug('Parsing XML file: %s' % filename)
             try:
                 return minidom.parse(filename)
             except expat.ExpatError, e:
@@ -270,8 +348,11 @@ class RegistryParser(log.Loggable):
     def _parseComponent(self, node):
         # <component type="..." base="...">
         #   <source>
+        #   <eater>
+        #   <feeder>
         #   <properties>
-        #   <entry>
+        #   <entries>
+        #   <synchronization>
         # </component>
         
         if not node.hasAttribute('type'):
@@ -294,6 +375,10 @@ class RegistryParser(log.Loggable):
         files = []
         source = None
         entries = {}
+        eaters = []
+        feeders = []
+        needs_sync = False
+        clock_priority = 100
         for child in self._getChildNodes(node):
             if child.nodeName == 'source':
                 source = self._parseSource(child)
@@ -303,12 +388,20 @@ class RegistryParser(log.Loggable):
                 files = self._parseFiles(child)
             elif child.nodeName == 'entries':
                 entries = self._parseEntries(child)
+            elif child.nodeName == 'eater':
+                eaters.append(self._parseEater(child))
+            elif child.nodeName == 'feeder':
+                feeders.append(self._parseFeeder(child))
+            elif child.nodeName == 'synchronization':
+                needs_sync, clock_priority = self._parseSynchronization(child)
             else:
                 raise XmlParserError("unexpected node: %s" % child)
 
         return RegistryEntryComponent(self.filename,
                                       type, source, baseDir,
-                                      properties.values(), files, entries)
+                                      properties, files,
+                                      entries, eaters, feeders,
+                                      needs_sync, clock_priority)
 
     def _parseSource(self, node):
         # <source location="..."/>
@@ -375,6 +468,7 @@ class RegistryParser(log.Loggable):
         # <entries>
         #   <entry type="" location="" function=""/>
         # </entries>
+        # returns: dict of type -> entry
 
         entries = {}
         for child in self._getChildNodes(node):
@@ -400,6 +494,41 @@ class RegistryParser(log.Loggable):
             
         return entries
 
+    def _parseEater(self, node):
+        # <eater name="..." [required="yes/no"] [multiple="yes/no"]/>
+        if not node.hasAttribute('name'):
+            raise XmlParserError("<eater> must have a name attribute")
+        name = str(node.getAttribute('name'))
+
+        required = True
+        if node.hasAttribute('required'):
+            required = _istrue(node.getAttribute('required'))
+        
+        multiple = False
+        if node.hasAttribute('multiple'):
+            multiple = _istrue(node.getAttribute('multiple'))
+
+        return RegistryEntryEater(name, required, multiple)
+
+    def _parseFeeder(self, node):
+        # <feeder name="..."/>
+        if not node.hasAttribute('name'):
+            raise XmlParserError("<feeder> must have a name attribute")
+
+        return str(node.getAttribute('name'))
+
+    def _parseSynchronization(self, node):
+        # <synchronization [required="yes/no"] [clock-priority="yes/no"]/>
+        required = False
+        if node.hasAttribute('required'):
+            required = _istrue(node.getAttribute('required'))
+        
+        clock_priority = 100
+        if node.hasAttribute('clock-priority'):
+            clock_priority = int(node.getAttribute('clock-priority'))
+
+        return required, clock_priority
+
     ## Component registry specific functions
     def parseRegistryFile(self, filename, string=None):
         # FIXME: better separation of filename and string ?
@@ -464,6 +593,13 @@ class RegistryParser(log.Loggable):
             raise XmlParserError, "<bundle> must have a name attribute"
         name = str(node.getAttribute('name'))
 
+        project = 'flumotion'
+        if node.hasAttribute('project'):
+            project = str(node.getAttribute('project'))
+        under = 'pythondir'
+        if node.hasAttribute('under'):
+            under = str(node.getAttribute('under'))
+
         dependencies = []
         directories = []
 
@@ -477,7 +613,7 @@ class RegistryParser(log.Loggable):
             else:
                 raise XmlParserError("<bundle> unexpected node: %s" % child.nodeName)
 
-        return RegistryEntryBundle(name, dependencies, directories)
+        return RegistryEntryBundle(name, project, under, dependencies, directories)
 
     def _parseBundleDependencies(self, node):
         # <dependencies>
@@ -516,7 +652,7 @@ class RegistryParser(log.Loggable):
 
     def _parseBundleDirectory(self, node):
         # <directory name="">
-        #   <filename location="">
+        #   <filename location="" [ relative="" ] >
         # </directory>
         
         filenames = []
@@ -611,11 +747,14 @@ class RegistryParser(log.Loggable):
 # FIXME: filename -> path
 class RegistryDirectory:
     """
-    I represent a directory under a registry path.
+    I represent a directory under a path managed by the registry.
+    I can be queried for a list of partial registry .xml files underneath
+    the given path, under the given prefix.
     """
-    def __init__(self, path):
+    def __init__(self, path, prefix='flumotion'):
         self._path = path
-        self._files = self._getFileList(self._path)
+        self._prefix = prefix
+        self._files = self._getFileList(os.path.join(path, prefix))
         
     def _getFileList(self, root):
         """
@@ -676,15 +815,19 @@ class ComponentRegistry(log.Loggable):
             os.access(self.filename, os.R_OK)):
             self.info('Parsing registry: %s' % self.filename)
             self._parser.parseRegistry(self.filename)
+
+        self.verify()
     
     def addFile(self, filename, string=None):
+        if filename.endswith('registry.xml'):
+            self.warning('%s seems to be an old registry in your tree, please remove it' % filename) 
         self.debug('Adding file: %s' % filename)
         self._parser.parseRegistryFile(filename, string)
         
     def addFromString(self, string):
         self.addFile('<string>', string)
         
-    def addRegistryPath(self, path, force=False):
+    def addRegistryPath(self, path, prefix='flumotion', force=False):
         """
         Add a registry path to this registry.
 
@@ -692,24 +835,31 @@ class ComponentRegistry(log.Loggable):
         if the directory has been modified since the last scan.
         If force is True, then the registry path will be parsed regardless
         of the modification time.
+
+        @param path: a full path containing a 'flumotion' directory,
+                     which will be scanned for registry files.
         """
+        self.debug('path %s, prefix %s, force %r' % (path, prefix, force))
         if not os.path.exists(path):
             return
 
         directory = self._parser._directories.get(path, None)
-        if not force:
+        if not force and directory:
             # if directory is watched in the registry, and it hasn't been
             # updated, everything's fine
-            if directory \
-                and directory.lastModified() < _getMTime(self.filename):
+            dTime = directory.lastModified()
+            fTime = _getMTime(self.filename)
+            if dTime < fTime:
+                self.debug('%s has not been changed since last registry parse' %
+                    path)
                 return
         
         # registry path was either not watched or updated, or a force was
         # asked, so reparse
         self.info('Scanning registry path %s' % path)
-        registryPath = RegistryDirectory(path)
+        registryPath = RegistryDirectory(path, prefix=prefix)
         files = registryPath.getFiles()
-        self.debug('Found %d files' % len(files))
+        self.debug('Found %d possible registry files' % len(files))
         map(self.addFile, files)
         
         self._parser.addDirectory(registryPath)
@@ -718,6 +868,9 @@ class ComponentRegistry(log.Loggable):
         return len(self._parser._components) == 0
 
     def getComponent(self, name):
+        """
+        @rtype: L{RegistryEntryComponent}
+        """
         return self._parser._components[name]
 
     def hasComponent(self, name):
@@ -732,6 +885,52 @@ class ComponentRegistry(log.Loggable):
     def getDirectories(self):
         return self._parser.getDirectories()
 
+    def makeBundlerBasket(self):
+        """
+        @rtype: L{flumotion.common.bundle.BundlerBasket}
+        """
+        def load():
+            ret = bundle.BundlerBasket()
+            for b in self.getBundles():
+                bundleName = b.getName()
+                self.debug('Adding bundle %s' % bundleName)
+                for d in b.getDirectories():
+                    directory = d.getName()
+                    for file in d.getFiles():
+                        try:
+                            basedir = b.getBaseDir()
+                        except errors.NoProjectError, e:
+                            self.warning("Could not find project %s" % e.args)
+                            raise
+                        fullpath = os.path.join(basedir, directory,
+                                                file.getLocation())
+                        relative = file.getRelative()
+                        self.log('Adding path %s as %s to bundle %s' % (
+                            fullpath, relative, bundleName))
+                        try:
+                            ret.add(bundleName, fullpath, relative)
+                        except Exception, e:
+                            self.debug("Reason: %r" % e)
+                            raise RuntimeError(
+                                'Could not add %s to bundle %s (%s)'
+                                % (fullpath, bundleName, e))
+                for d in b.getDependencies():
+                    self.log('Adding dependency of %s on %s' % (bundleName, d))
+                    ret.depend(bundleName, d)
+            return ret
+
+        try:
+            return load()
+        except Exception, e:
+            self.warning("Bundle problem, rebuilding registry (%s)" % e)
+            self.verify(force=True)
+            try:
+                return load()
+            except Exception, e:
+                self.debug("Could not register bundles twice: %s" %
+                    log.getExceptionMessage(e))
+                self.error("Could not not register bundles (%s)" % e)
+
     def dump(self, fd):
         """
         Dump the cache of components to the given opened file descriptor.
@@ -744,14 +943,25 @@ class ComponentRegistry(log.Loggable):
             print >> fd, ' '*i + msg
             
         w(0, '<registry>')
-        
+        w(0, '')
+
         # Write components
         w(2, '<components>')
+        w(0, '')
         for component in self.getComponents():
             w(4, '<component type="%s" base="%s">' % (component.getType(),
                 component.getBase()))
 
             w(6, '<source location="%s"/>' % component.getSource())
+            for x in component.getEaters():
+                w(6, '<eater name="%s" required="%s" multiple="%s"/>'
+                  % (x.getName(), x.getRequired() and "yes" or "no",
+                     x.getMultiple() and "yes" or "no"))
+            for x in component.getFeeders():
+                w(6, '<feeder name="%s"/>' % x)
+            w(6, '<synchronization required="%s" clock-priority="%d"/>'
+              % (component.getNeedsSynchronization() and "yes" or "no",
+                 component.getClockPriority()))
             w(6, '<properties>')
             for prop in component.getProperties():
                 w(8, '<property name="%s" type="%s" required="%s" multiple="%s"/>' % (
@@ -780,13 +990,16 @@ class ComponentRegistry(log.Loggable):
                         entry.getFunction()))
                 w(6, '</entries>')
             w(4, '</component>')
+            w(0, '')
                 
         w(2, '</components>')
+        w(0, '')
 
         # bundles
         w(2, '<bundles>')
         for bundle in self.getBundles():
-            w(4, '<bundle name="%s">' % bundle.getName())
+            w(4, '<bundle name="%s" project="%s">' % (
+                bundle.getName(), bundle.getProject()))
 
             dependencies = bundle.getDependencies()
             if dependencies:
@@ -807,6 +1020,7 @@ class ComponentRegistry(log.Loggable):
                 w(6, '</directories>')
                 
             w(4, '</bundle>')
+            w(0, '')
         w(2, '</bundles>')
 
 
@@ -814,9 +1028,11 @@ class ComponentRegistry(log.Loggable):
         directories = self.getDirectories()
         if directories:
             w(2, '<directories>')
+            w(0, '')
             for d in directories:
                 w(4, '<directory filename="%s"/>' % d.getPath())
             w(2, '</directories>')
+            w(0, '')
         
         w(0, '</registry>')
 
@@ -867,20 +1083,18 @@ class ComponentRegistry(log.Loggable):
             else:
                 raise
 
-    def verify(self):
+    def verify(self, force=False):
         """
         Verify if the registry is uptodate and rebuild if it is not.
+
+        @param force: True if the registry needs rebuilding for sure.
         """
-        force = False # set to True if needs rebuilding
-        
         # construct a list of all paths to scan for registry .xml files
-        registryPaths = [os.path.join(configure.pythondir, 'flumotion'), ]
-        if os.environ.has_key('FLU_REGISTRY_PATH'):
-            paths = os.environ['FLU_REGISTRY_PATH']
+        registryPaths = [configure.pythondir, ]
+        if os.environ.has_key('FLU_PROJECT_PATH'):
+            paths = os.environ['FLU_PROJECT_PATH']
             registryPaths += paths.split(':')
         
-        self.debug('registry paths: %s' % ", ".join(registryPaths))
-
         # get the list of all paths used to construct the old registry
         oldRegistryPaths = [dir.getPath()
                                   for dir in self.getDirectories()]
@@ -892,15 +1106,32 @@ class ComponentRegistry(log.Loggable):
         registryPaths.sort()
         oldRegistryPaths.sort()
         if registryPaths != oldRegistryPaths:
+            self.debug('old and new registry paths are different')
             self.info('Rescanning registry paths')
             force = True
+        else:
+            self.debug('registry paths are still the same')
 
         if force:
             self.clean()
         
         for directory in registryPaths:
-            self.addRegistryPath(directory, force)
+            self.addRegistryPath(directory, force=force)
 
         self.save(force)
 
-registry = ComponentRegistry()
+__registry = None
+
+def getRegistry():
+    """
+    Return the registry.  Only one registry will ever be created.
+
+    @rtype: L{ComponentRegistry}
+    """
+    global __registry
+
+    if not __registry:
+        log.debug('registry', 'instantiating registry')
+        __registry = ComponentRegistry()
+
+    return __registry
