@@ -25,8 +25,6 @@ Base classes for component UI's using GTK+
 
 import os
 
-import gettext
-
 import gtk
 import gtk.glade
 
@@ -36,6 +34,7 @@ from twisted.internet import defer
 from flumotion.common import errors, log, common
 from flumotion.twisted import flavors
 from flumotion.twisted.defer import defer_generator_method
+from flumotion.twisted.compat import implements
 
 class BaseAdminGtk(log.Loggable):
     """
@@ -46,7 +45,7 @@ class BaseAdminGtk(log.Loggable):
     @ivar nodes: an ordered dict of name -> L{BaseAdminGtkNode}
     """
 
-    __implements__ = (flavors.IStateListener,)
+    implements(flavors.IStateListener)
 
     logCategory = "admingtk"
     gettext_domain = 'flumotion'
@@ -327,14 +326,14 @@ class EffectAdminGtkNode(BaseAdminGtkNode):
     I am a view on a set of properties for an effect on a component.
 
     """
-    def __init__(self, state, admin, effectName):
+    def __init__(self, state, admin, effectName, title=None):
         """
         @param state: state of component this is a UI for
         @type  state: L{flumotion.common.planet.AdminComponentState}
         @param admin: the admin model that interfaces with the manager for us
         @type  admin: L{flumotion.admin.admin.AdminModel}
         """
-        BaseAdminGtkNode.__init__(self, state, admin)
+        BaseAdminGtkNode.__init__(self, state, admin, title)
         self.effectName = effectName
 
     def effectCallRemote(self, methodName, *args, **kwargs):
