@@ -39,6 +39,11 @@ class BouncerMedium(component.BaseComponentMedium):
 
     logCategory = 'bouncermedium'
     def remote_authenticate(self, keycard):
+        """
+        Authenticates the given keycard.
+
+        @type  keycard: L{flumotion.common.keycards.Keycard}
+        """
         return self.comp.authenticate(keycard)
 
     def remote_removeKeycardId(self, keycardId):
@@ -89,6 +94,13 @@ class Bouncer(component.BaseComponent):
          
     # FIXME: do we need this at all in the base class ?
     def authenticate(self, keycard):
+        """
+        Authenticate the given keycard.
+        Can be overridden by subclasses.
+        Return the keycard with state AUTHENTICATED to authenticate,
+        with state REQUESTING to continue the authentication process,
+        or None to deny the keycard.
+        """
         if not components.implements(keycard, credentials.ICredentials):
             self.warning('keycard %r does not implement ICredentials', keycard)
             raise AssertionError
