@@ -26,6 +26,7 @@ an htpasswd-backed bouncer with crypt passwords
 import md5
 import random
 
+# Twisted recommends you to use zope.interface directly
 from twisted.python import components
 from twisted.cred import error
 from twisted.internet import defer
@@ -66,14 +67,14 @@ class HTPasswdCrypt(bouncer.Bouncer):
             self._data = props['data']
             log.debug('htpasswd', 'using in-line data for passwords')
         else:
-            raise config.ConfigError(
-                'HTPasswdCrypt config needs either a <data> or <filename> entry')
+            return defer.fail(config.ConfigError(
+                'HTPasswdCrypt needs either a <data> or <filename> entry'))
         # FIXME: generalize to a start method, possibly linked to mood
         if self._filename:
             try:
                 lines = open(self._filename).readlines()
             except IOError, e:
-                raise config.ConfigError(str(e))
+                return defer.fail(config.ConfigError(str(e)))
         else:
             lines = self._data.split("\n")
 
@@ -111,7 +112,7 @@ class HTPasswdCrypt(bouncer.Bouncer):
     
     def authenticate(self, keycard):
         # FIXME: move checks up in the base class ?
-        if not components.implements(keycard, credentials.IUsernameCryptPassword):
+        if not compat.implementsInterface(keycard, credentials.IUsernameCryptPassword):
             self.warning('keycard %r does not implement IUsernameCryptPassword' % keycard)
         if not self.typeAllowed(keycard):
             self.warning('keycard %r not in type list %r' % (keycard, self.keycardClasses))
@@ -150,6 +151,7 @@ class HTPasswdCrypt(bouncer.Bouncer):
                 del self._challenges[keycard.id]
 
         # use the checker
+        self.debug('submitting keycard %r to checker' % keycard)
         d = self._checker.requestAvatarId(keycard)
         d.addCallback(self._requestAvatarIdCallback, keycard)
         d.addErrback(self._requestAvatarIdErrback, keycard)
