@@ -40,6 +40,7 @@ from flumotion.common.planet import moods
 from flumotion.configure import configure
 from flumotion.twisted import credentials
 from flumotion.twisted import pb as fpb
+from flumotion.twisted.compat import implements
 from flumotion.common.pygobject import gsignal
 
 
@@ -62,7 +63,8 @@ class ComponentClientFactory(fpb.ReconnectingFPBClientFactory):
 
         self.maxDelay = 10
         # get the interfaces implemented by the component medium class
-        self.interfaces = self.medium.__class__.__implements__
+        # FIXME: interface
+        #self.interfaces = self.medium.__class__.__implements__
 
         self.logName = component.name
         
@@ -124,7 +126,7 @@ class BaseComponentMedium(medium.BaseMedium):
     I have a remote reference to the manager's avatar to call upon.
     """
 
-    __implements__ = interfaces.IComponentMedium,
+    implements(interfaces.IComponentMedium)
     logCategory = 'basecompmedium'
 
     def __init__(self, component):
@@ -335,6 +337,8 @@ class BaseComponent(common.InitMixin, log.Loggable, gobject.GObject):
         Subclasses can implement me to set up the component before it is
         started.  It should set up the component, possibly opening files
         and resources.
+        Non-programming errors should not be raised, but returned as a
+        failing deferred.
 
         self.config will be set when this is called.
 
