@@ -43,7 +43,7 @@ class StatisticsAdminGtkNode(BaseAdminGtkNode):
         self._stats = None
         self._hasgnomevfs = False
         try:
-            import gnomevfs
+            __import__('gnomevfs')
             self._hasgnomevfs = True
         except:
             pass
@@ -90,8 +90,8 @@ class StatisticsAdminGtkNode(BaseAdminGtkNode):
         peakTime = state.get('clients-peak-time')
         if not isinstance(peakTime, str):
             peakTime = time.strftime ("%c", time.localtime(peakTime))
-            
-            self.debug('Converted peak time to %s' % peakTime)
+            self.log('Converted peak time to %s' % peakTime)
+
         self.labels['clients-peak-time'].set_text(peakTime)
         
         for name in self.labels.keys():
