@@ -28,7 +28,7 @@ import gst
 # socket needed to get hostname
 import socket
 
-from twisted.internet import reactor, error
+from twisted.internet import reactor, error, defer
 from twisted.web import server
 
 from flumotion.component import feedcomponent
@@ -261,18 +261,51 @@ class MultifdSinkStreamer(feedcomponent.ParseLaunchComponent, Stats):
         
         # FIXME: tie these together more nicely
         self.resource = resources.HTTPStreamingResource(self)
+
+        sink = self.get_element('sink')
+
+        # check how to set client sync mode
+        self.burst_on_connect = properties.get('burst_on_connect', False)
+        if gstreamer.element_factory_has_property('multifdsink', 'sync-method'):
+            self.debug("multifdsink has new sync-method property")
+            if self.burst_on_connect:
+                self.debug("burst-on-connect, setting sync-method 2")
+                sink.set_property('sync-method', 2)
+            else:
+                self.debug("no burst-on-connect, setting sync-method 0")
+                sink.set_property('sync-method', 0)
+        else:
+            # old property; does sync-to-keyframe
+            self.debug("multifdsink has old sync-clients property")
+            sink.set_property('sync-clients', self.burst_on_connect)
+            
+        # FIXME: these should be made threadsafe if we use GstThreads
+        sink.connect('deep-notify::caps', self._notify_caps_cb)
+
+        if gst.gst_version < (0, 9):
+            def sink_state_change_cb(element, old, state):
+                # when our sink is PLAYING, then we are HAPPY
+                # FIXME: add more moods
+                if state == gst.STATE_PLAYING:
+                    self.debug('Ready to serve clients')
+                    self.setMood(moods.happy)
+
+            sink.connect('state-change', sink_state_change_cb)
+
+        # these are made threadsafe using idle_add in the handler
+        sink.connect('client-removed', self._client_removed_cb)
+        sink.connect('client-added', self._client_added_cb)
+
         
         if properties.has_key('logfile'):
             file = properties['logfile']
             self.debug('Logging to %s' % file)
             try:
                 self.resource.setLogfile(file)
-            except IOError, data:
+            except IOError, e:
                 raise errors.PropertiesError(
                     'could not open log file %s for writing (%s)' % (
-                        file, data[1]))
-
-        self.burst_on_connect = properties.get('burst_on_connect', False)
+                        file, e.strerror))
 
         if properties.has_key('user_limit'):
             self.resource.setUserLimit(int(properties['user_limit']))
@@ -424,36 +457,6 @@ class MultifdSinkStreamer(feedcomponent.ParseLaunchComponent, Stats):
 
     ### END OF THREAD-AWARE CODE
 
-    def link_setup(self, eaters, feeders):
-        sink = self.get_element('sink')
-
-        # check how to set client sync mode
-        if gstreamer.element_factory_has_property('multifdsink', 'sync-method'):
-            if self.burst_on_connect:
-                sink.set_property('sync-method', 2)
-            else:
-                sink.set_property('sync-method', 0)
-        else:
-            # old property; does sync-to-keyframe
-            sink.set_property('sync-clients', self.burst_on_connect)
-            
-        # FIXME: these should be made threadsafe if we use GstThreads
-        sink.connect('deep-notify::caps', self._notify_caps_cb)
-
-        if gst.gst_version < (0, 9):
-            def sink_state_change_cb(element, old, state):
-                # when our sink is PLAYING, then we are HAPPY
-                # FIXME: add more moods
-                if state == gst.STATE_PLAYING:
-                    self.debug('Ready to serve clients')
-                    self.setMood(moods.happy)
-
-            sink.connect('state-change', sink_state_change_cb)
-
-        # these are made threadsafe using idle_add in the handler
-        sink.connect('client-removed', self._client_removed_cb)
-        sink.connect('client-added', self._client_added_cb)
-
     def start(self, *args, **kwargs):
         root = resources.HTTPRoot()
         root.putChild(self.mountPoint, self.resource)
@@ -463,10 +466,12 @@ class MultifdSinkStreamer(feedcomponent.ParseLaunchComponent, Stats):
             reactor.listenTCP(self.port, server.Site(resource=root))
             feedcomponent.ParseLaunchComponent.start(self, *args, **kwargs)
         except error.CannotListenError:
-            self.warning('Port %d is not available.' % self.port)
+            t = 'Port %d is not available.' % self.port
+            self.warning(t)
             m = messages.Error(T_(N_(
                 "Network error: TCP port %d is not available."), self.port))
             self.addMessage(m)
             self.setMood(moods.sad)
+            return defer.fail(errors.ComponentStartHandledError(t))
 
 pygobject.type_register(MultifdSinkStreamer)
