@@ -1,11 +1,9 @@
 # -*- Mode: Python; test-case-name: flumotion.test.test_http -*-
 # vi:si:et:sw=4:sts=4:ts=4
 #
-# flumotion/component/consumers/httpstreamer/resources.py:
-# web server part of http streamer component
-#
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005 Fluendo, S.L. (www.fluendo.com). All rights reserved.
+# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
 # the GNU General Public License version 2 as published by
@@ -28,9 +26,15 @@ import resource
 
 import gst
 
-from twisted.protocols import http
+try:
+    from twisted.web import http
+except ImportError:
+    from twisted.protocols import http
+
 from twisted.web import server, resource as web_resource
 from twisted.internet import reactor, defer
+from twisted.python import reflect
+
 from flumotion.configure import configure
 from flumotion.common import errors
 
@@ -52,8 +56,58 @@ ERROR_TEMPLATE = """<!doctype html public "-//IETF//DTD HTML 2.0//EN">
 </html>
 """
 
-HTTP_VERSION = '%s/%s' % (HTTP_NAME, HTTP_VERSION)
+HTTP_SERVER = '%s/%s' % (HTTP_NAME, HTTP_VERSION)
 
+### This is new Issuer code that eventually should move to e.g.
+### flumotion.common.keycards or related
+
+class Issuer(log.Loggable):
+    """
+    I am a base class for all Issuers.
+    An issuer issues keycards of a given class based on an object
+    (incoming HTTP request, ...)
+    """
+    def issue(self, *args, **kwargs):
+        """
+        Return a keycard, or None, based on the given arguments. 
+        """
+        raise NotImplementedError
+
+class HTTPAuthIssuer(Issuer):
+    """
+    I create L{flumotion.common.keycards.KeycardUACPP} keycards based on
+    an incoming L{twisted.protocols.http.Request} request's standard
+    HTTP authentication information.
+    """
+    def issue(self, request):
+        # for now, we're happy with a UACPP keycard; the password arrives
+        # plaintext anyway
+        keycard = keycards.KeycardUACPP(
+            request.getUser(),
+            request.getPassword(), request.getClientIP())
+        self.debug('Asking for authentication, user %s, password %s, ip %s' % (
+            keycard.username, keycard.password, keycard.address))
+        return keycard
+ 
+class HTTPTokenIssuer(Issuer):
+    """
+    I create L{flumotion.common.keycards.KeycardToken} keycards based on
+    an incoming L{twisted.protocols.http.Request} request's GET "token"
+    parameter.
+    """
+    def issue(self, request):
+        if not 'token' in request.args.keys():
+            return None
+
+        # args can have lists as values, if more than one specified
+        token = request.args['token']
+        if not isinstance(token, str):
+            token = token[0]
+        
+        keycard = keycards.KeycardToken(token,
+            request.getClientIP())
+        return keycard
+ 
 ### the Twisted resource that handles the base URL
 class HTTPStreamingResource(web_resource.Resource, log.Loggable):
 
@@ -70,16 +124,18 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         """
         @param streamer: L{MultifdSinkStreamer}
         """
-        self.logfile = None
+        self._logfile = None
+        self._logfilename = None
             
         streamer.connect('client-removed', self._streamer_client_removed_cb)
         self.streamer = streamer
         
-        self._requests = {}         # request fd -> Request
-        self._fdToKeycard = {}      # request fd -> Keycard
-        self._idToKeycard = {}      # keycard id -> Keycard
-        self._fdToDurationCall = {} # request fd -> IDelayedCall for duration
-        self._domain = None         # used for auth challenge and on keycard
+        self._requests = {}            # request fd -> Request
+        self._fdToKeycard = {}         # request fd -> Keycard
+        self._idToKeycard = {}         # keycard id -> Keycard
+        self._fdToDurationCall = {}    # request fd -> IDelayedCall for duration
+        self._domain = None            # used for auth challenge and on keycard
+        self._issuer = HTTPAuthIssuer() # issues keycards; default for compat
         self.bouncerName = None
         self.requesterName = streamer.getName() # avatarId of streamer component
         
@@ -88,6 +144,8 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         web_resource.Resource.__init__(self)
 
     def _streamer_client_removed_cb(self, streamer, sink, fd, reason, stats):
+        # this is the callback attached to our flumotion component,
+        # not the GStreamer element
         try:
             request = self._requests[fd]
             self._removeClient(request, fd, stats)
@@ -97,9 +155,21 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
     def setRoot(self, path):
         self.putChild(path, self)
         
-    def setLogfile(self, logfile):
-        self.logfile = open(logfile, 'a')
+    def setLogfile(self, logfilename):
+        self._logfilename = logfilename
+        self._logfile = open(logfilename, 'a')
 
+    def rotateLogfile(self):
+        """
+        Close the logfile, then reopen using the previous logfilename
+        """
+        self.debug('rotating log file %s' % self._logfile)
+
+        if self._logfile:
+            self._logfile.close()
+        if self._logfilename:
+            self.setLogfile(self._logfilename)
+            
     def setDomain(self, domain):
         """
         Set a domain name on the resource, used in HTTP auth challenges and
@@ -132,7 +202,7 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         # time connected
         ident = '-'
         username = '-'
-        date = time.strftime('%d/%b/%Y:%H:%M:%S %z', time.localtime())
+        date = time.strftime('%d/%b/%Y:%H:%M:%S +0000', time.gmtime())
         request_str = '%s %s %s' % (request.method,
                                     request.uri,
                                     request.clientproto)
@@ -145,14 +215,14 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
                         time_connected)
         # make streamer notify manager of this msg
         self.streamer.sendLog(msg)
-        if not self.logfile:
+        if not self._logfile:
             return
-        self.logfile.write(msg)
-        self.logfile.flush()
+        self._logfile.write(msg)
+        self._logfile.flush()
 
-    def setMaxClients(self, maxclients):
-        self.info('setting maxclients to %d' % maxclients)
-        self.maxclients = maxclients
+    def setUserLimit(self, limit):
+        self.info('setting maxclients to %d' % limit)
+        self.maxclients = limit
 
     def setBouncerName(self, bouncerName):
         self.bouncerName = bouncerName
@@ -160,6 +230,16 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
     def setRequesterName(self, requesterName):
         self.requesterName = requesterName
 
+    def setIssuerClass(self, issuerClass):
+        # FIXME: in the future, we want to make this pluggable and have it
+        # look up somewhere ?
+        if issuerClass == 'HTTPTokenIssuer':
+            self._issuer = HTTPTokenIssuer()
+        elif issuerClass == 'HTTPAuthIssuer':
+            self._issuer = HTTPAuthIssuer()
+        else:
+            raise ValueError, "issuerClass %s not accepted" % issuerClass
+
     # FIXME: rename to writeHeaders
     """
     Write out the HTTP headers for the incoming HTTP request.
@@ -187,11 +267,19 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
             headers.append('%s: %s\r\n' % (field, name))
 
         # Mimic Twisted as close as possible
-        setHeader('Server', HTTP_VERSION)
+        content = self.streamer.get_content_type()
+        setHeader('Server', HTTP_SERVER)
         setHeader('Date', http.datetimeToString())
         setHeader('Cache-Control', 'no-cache')
         setHeader('Cache-Control', 'private')
-        setHeader('Content-type', self.streamer.get_content_type())
+        setHeader('Content-type', content)
+        
+        # ASF needs a Pragma header for live broadcasts
+        if content in [
+            "video/x-ms-asf",
+            "audio/x-ms-asf",
+        ]:
+            setHeader('Pragma', 'features=broadcast')
             
         #self.debug('setting Content-type to %s' % mime)
         ### FIXME: there's a window where Twisted could have removed the
@@ -224,11 +312,12 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         maximum number of allowed clients based on soft limit for number of
         open file descriptors and fd reservation
         """
-        if self.maxclients != -1:
+        limit = resource.getrlimit(resource.RLIMIT_NOFILE)
+        fd_limit = limit[0] - self.__reserve_fds__
+        if self.maxclients != -1 and fd_limit >= self.maxclients:
             return self.maxclients
         else:
-            limit = resource.getrlimit(resource.RLIMIT_NOFILE)
-            return limit[0] - self.__reserve_fds__
+            return fd_limit
 
     def reachedMaxClients(self):
         return len(self._requests) >= self.maxAllowedClients()
@@ -237,20 +326,20 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         """
         Returns: a deferred returning a keycard or None
         """
-        # for now, we're happy with a UACPP keycard; the password arrives
-        # plaintext anyway
-        keycard = keycards.KeycardUACPP(
-            request.getUser(),
-            request.getPassword(), request.getClientIP())
+        keycard = self._issuer.issue(request)
+        if not keycard:
+            self.debug('no keycard from issuer, firing None')
+            return defer.succeed(None)
+
         keycard.requesterName = self.requesterName
         keycard._fd = request.transport.fileno()
         
         if self.bouncerName == None:
+            self.debug('no bouncer, accepting')
             return defer.succeed(keycard)
 
-        self.debug('Asking for authentication, user %s, password %s, ip %s' % (
-            keycard.username, keycard.password, keycard.address))
         keycard.setDomain(self._domain)
+        self.debug('sending keycard to bouncer %r' % self.bouncerName)
         return self.streamer.medium.authenticate(self.bouncerName, keycard)
 
     def _addClient(self, request):
@@ -302,7 +391,7 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
             del self._idToKeycard[id]
             self.debug('[fd %5d] asking bouncer %s to remove keycard id %s' % (
                 fd, self.bouncerName, id))
-            self.streamer.medium.removeKeycard(self.bouncerName, id)
+            self.streamer.medium.removeKeycardId(self.bouncerName, id)
         if self._fdToDurationCall.has_key(fd):
             self.debug('[fd %5d] canceling later expiration call' % fd)
             self._fdToDurationCall[fd].cancel()
@@ -376,12 +465,10 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         elif self.reachedMaxClients():
             return self._handleMaxClients(request)
 
+        self.debug('_render(): asked for (possible) authentication')
         d = self.authenticate(request)
         d.addCallback(self._authenticatedCallback, request)
         d.addErrback(self._authenticatedErrback, request)
-        self.debug('_render(): asked for authentication')
-        # FIXME
-        #d.addErrback()
 
         # we MUST return this from our _render.
         # FIXME: check if this is true
@@ -493,7 +580,6 @@ class HTTPStreamingResource(web_resource.Resource, log.Loggable):
         ip = request.getClientIP()
 
         self.info('[fd %5d] Started streaming to %s' % (fd, ip))
-
  
     render_GET = _render
     render_HEAD = _render
