@@ -72,6 +72,11 @@ class FeedComponent(basecomponent.BaseComponent):
         self.feed_names = []
         self.feeder_names = []
 
+        # statechange -> [ deferred ]
+        self._stateChangeDeferreds = {}
+        # defer to fireback when master clock has been set
+        self._masterClockDefer = None
+
     def do_setup(self):
         eater_config = self.config.get('source', [])
         feeder_config = self.config.get('feed', [])
@@ -188,6 +193,32 @@ class FeedComponent(basecomponent.BaseComponent):
     def get_pipeline(self):
         return self.pipeline
 
+    def _addStateChangeDeferred(self, statechange):
+        if statechange not in self._stateChangeDeferreds:
+            self._stateChangeDeferreds[statechange] = []
+
+        d = defer.Deferred()
+        self._stateChangeDeferreds[statechange].append(d)
+
+        return d
+
+    # GstPython should have something for this, but doesn't.
+    def _getStateChange(self, old, new):
+        if old == gst.STATE_NULL and new == gst.STATE_READY:
+            return gst.STATE_CHANGE_NULL_TO_READY
+        elif old == gst.STATE_READY and new == gst.STATE_PAUSED:
+            return gst.STATE_CHANGE_READY_TO_PAUSED
+        elif old == gst.STATE_PAUSED and new == gst.STATE_PLAYING:
+            return gst.STATE_CHANGE_PAUSED_TO_PLAYING
+        elif old == gst.STATE_PLAYING and new == gst.STATE_PAUSED:
+            return gst.STATE_CHANGE_PLAYING_TO_PAUSED
+        elif old == gst.STATE_PAUSED and new == gst.STATE_READY:
+            return gst.STATE_CHANGE_PAUSED_TO_READY
+        elif old == gst.STATE_READY and new == gst.STATE_NULL:
+            return gst.STATE_CHANGE_READY_TO_NULL
+        else:
+            return 0
+
        
     def bus_watch_func(self, bus, message):
         t = message.type
@@ -202,6 +233,14 @@ class FeedComponent(basecomponent.BaseComponent):
                     % (src, old.value_nick, new.value_nick)) 
                 if old == gst.STATE_PAUSED and new == gst.STATE_PLAYING:
                     self.setMood(moods.happy)
+
+                change = self._getStateChange(old,new)
+                if change in self._stateChangeDeferreds:
+                    dlist = self._stateChangeDeferreds[change]
+                    for d in dlist:
+                        d.callback(None)
+                    del self._stateChangeDeferreds[change]
+
             elif src.get_name() in ['feeder:'+n for n in self.feeder_names]:
                 if old == gst.STATE_PAUSED and new == gst.STATE_PLAYING:
                     self.debug('feeder %s is now feeding' % src.get_name())
@@ -338,8 +377,10 @@ class FeedComponent(basecomponent.BaseComponent):
         return defer.succeed(None)
 
     def set_master_clock(self, ip, port, base_time):
-        clock = gst.NetClientClock(None, ip, port, base_time)
-        self.pipeline.set_base_time(base_time)
+        # GstNetClientClock.__init__() argument 4 must be long, not int
+        clock = gst.NetClientClock(None, ip, port, long(base_time))
+        # GstElement.set_base_time() argument 1 must be long, not int
+        self.pipeline.set_base_time(long(base_time))
         self.pipeline.use_clock(clock)
 
     def provide_master_clock(self, port):
@@ -348,6 +389,33 @@ class FeedComponent(basecomponent.BaseComponent):
 
         @returns: (ip, port, base_time) triple.
         """
+        self._masterClockDefer = defer.Deferred()
+        self._masterClockPort = port
+        return self._masterClockDefer
+
+    def really_provide_master_clock(self):
+        def pipelinePaused(r):
+            clock = self.pipeline.get_clock()
+            # make sure the pipeline sticks with this clock
+            self.pipeline.use_clock(clock)
+
+            self.clock_provider = gst.NetTimeProvider(clock, None, 
+                self._masterClockPort)
+            # small window here but that's ok
+            self.clock_provider.set_property('active', False)
+        
+            base_time = clock.get_time()
+            self.pipeline.set_base_time(base_time)
+
+            self.debug('provided master clock from %r, base time %s'
+                       % (clock, gst.TIME_ARGS(base_time)))
+
+            # FIXME: this is always localhost, no ? Not sure if this is useful
+            ip = self.state.get('ip')
+
+            self._masterClockDefer.callback((ip, self._masterClockPort, 
+                base_time))
+
         if not self.pipeline:
             self.warning('No self.pipeline, cannot provide master clock')
             # FIXME: should we have a NoSetupError() for cases where setup
@@ -357,24 +425,14 @@ class FeedComponent(basecomponent.BaseComponent):
             self.warning('already had a clock provider, removing it')
             self.clock_provider = None
 
-        clock = self.pipeline.get_clock()
-        # make sure the pipeline sticks with this clock
-        self.pipeline.use_clock(clock)
+        # We need to be >= PAUSED to get the correct clock, in general
+        self.info ("Setting pipeline to PAUSED")
 
-        self.clock_provider = gst.NetTimeProvider(clock, None, port)
-        # small window here but that's ok
-        self.clock_provider.set_property('active', False)
-        
-        base_time = clock.get_time()
-        self.pipeline.set_base_time(base_time)
+        d = self._addStateChangeDeferred(gst.STATE_CHANGE_READY_TO_PAUSED)
+        self.pipeline.set_state(gst.STATE_PAUSED)
 
-        self.debug('provided master clock from %r, base time %s'
-                   % (clock, gst.TIME_ARGS(base_time)))
-
-        # FIXME: this is always localhost, no ? Not sure if this is useful
-        ip = self.state.get('ip')
-
-        return (ip, port, base_time)
+        d.addCallback(pipelinePaused)
+        return d
 
     # FIXME: rename the comment, unambiguate it, and comment on it
     # FIXME: rename, unambiguate and comment
@@ -396,13 +454,15 @@ class FeedComponent(basecomponent.BaseComponent):
 
         self.debug('setting up feeders')
         self._setup_feeders(feedersData)
-        
-        # call a child's link_setup() method if it has it
-        func = getattr(self, 'link_setup', None)
-        if func:
-            self.debug('calling function %r' % func)
-            func(eatersData, feedersData)
-            
+
+        if self._masterClockDefer:
+            d = self.really_provide_master_clock()
+            d.addCallback(self.really_link)
+            return d
+        else:
+            return self.really_link()
+
+    def really_link(self, res=None):
         self.debug('setting pipeline to playing')
 
         if self.clock_provider:
