@@ -2,7 +2,8 @@
 # vi:si:et:sw=4:sts=4:ts=4
 #
 # Flumotion - a streaming media server
-# Copyright (C) 2004,2005 Fluendo, S.L. (www.fluendo.com). All rights reserved.
+# Copyright (C) 2004,2005,2006 Fluendo, S.L. (www.fluendo.com).
+# All rights reserved.
 
 # This file may be distributed and/or modified under the terms of
 # the GNU General Public License version 2 as published by
@@ -18,21 +19,22 @@
 
 # Headers in this file shall remain intact.
 
+import gst
+
 from flumotion.component import feedcomponent
 
 class Multipart(feedcomponent.ParseLaunchComponent):
-    def __init__(self, name, sources, pipeline):
-        feedcomponent.ParseLaunchComponent.__init__(self, name,
-                                                    sources,
-                                                    ['default'],
-                                                    pipeline)
+    def get_pipeline_string(self, properties):
+        sources = self.config['source']
 
-def createComponent(config):
-    pipeline = 'multipartmux name=muxer '
-    for eater in config['source']:
-        pipeline += '{ @ eater:%s @ ! queue } ! muxer. ' % eater
-    pipeline += 'muxer.'
-    
-    component = Multipart(config['name'], config['source'], pipeline)
-    
-    return component
+        pipeline = 'multipartmux name=muxer '
+        for eater in sources:
+            if gst.gst_version < (0, 9):
+                tmpl = '{ @ eater:%s @ ! queue } ! muxer. '
+            else:
+                tmpl = '@ eater:%s @ ! queue ! muxer. '
+            pipeline += tmpl % eater
+
+        pipeline += 'muxer.'
+
+        return pipeline
