@@ -26,6 +26,7 @@ def get():
         homedir = os.environ['HOME']
         registrydir = os.path.join(homedir, '.flumotion')
         cachedir = os.path.join(homedir, '.flumotion')
+        daemondir = homedir
         def ensure(path):
             try:
                 os.listdir(path)
@@ -34,8 +35,11 @@ def get():
         ensure(registrydir)
         ensure(cachedir)
     except KeyError:
-        registrydir = os.path.join('/usr/local/var', 'cache', 'flumotion')
-        cachedir = os.path.join('/usr/local/var', 'cache', 'flumotion')
+        registrydir = os.path.join('/usr/local/var', 'cache', 'flumotion',
+            '.flumotion')
+        cachedir = os.path.join('/usr/local/var', 'cache', 'flumotion',
+            '.flumotion')
+        daemondir = os.path.join('/usr/local/var', 'cache', 'flumotion')
     except OSError, e:
         print 'Error making dir %s: %s' % (e.filename, e.strerror)
         print 'Things are probably going to break!'
@@ -44,6 +48,7 @@ def get():
         'isinstalled':   True,
         'cachedir':      cachedir,
         'configdir':     os.path.join('/usr/local/etc', 'flumotion'),
+        'daemondir':     daemondir,
         'gladedir':      os.path.join('/usr/local/share', 'flumotion', 'glade'),
         'imagedir':      os.path.join('/usr/local/share', 'flumotion', 'image'),
         'localedatadir': os.path.join('/usr/local/share'),
@@ -51,5 +56,5 @@ def get():
         'pythondir':     os.path.join('/usr/local/lib', 'flumotion', 'python'),
         'registrydir':   registrydir,
         'rundir':        os.path.join('/usr/local/var', 'run', 'flumotion'),
-        'version':       '0.2.1',
+        'version':       '0.2.2',
     }
