@@ -25,7 +25,7 @@ manager-side objects to handle administrative clients
 
 import os
 
-from twisted.internet import reactor
+from twisted.internet import reactor, defer
 from twisted.spread import pb
 from twisted.python import failure
 
@@ -38,6 +38,8 @@ from flumotion.common import messages
 # make ComponentState proxyable
 from flumotion.twisted import flavors
 
+from flumotion.twisted.compat import implements
+
 from flumotion.common import componentui
 
 # FIXME: rename to Avatar since we are in the admin. namespace ?
@@ -66,7 +68,7 @@ class AdminAvatar(base.ManagerAvatar):
         The list gets serialized to a list of
         L{flumotion.common.planet.AdminComponentState}
         
-        @rtype: C{list} of L{flumotion.common.planet.ManagerComponentState}
+        @rtype: list of L{planet.ManagerComponentState}
         """
         return self.vishnu.getComponentStates()
 
@@ -81,14 +83,29 @@ class AdminAvatar(base.ManagerAvatar):
         
     ### pb.Avatar IPerspective methods
     def perspective_getPlanetState(self):
+        """
+        Get the planet state.
+
+        @rtype: L{flumotion.common.planet.ManagerPlanetState}
+        """
         self.debug("returning planet state %r" % self.vishnu.state)
         return self.vishnu.state
 
     def perspective_getWorkerHeavenState(self):
+        """
+        Get the worker heaven state.
+
+        @rtype: L{flumotion.common.worker.ManagerWorkerHeavenState}
+        """
         self.debug("returning worker heaven state %r" % self.vishnu.state)
         return self.vishnu.workerHeaven.state
 
     def perspective_shutdown(self):
+        """
+        Shut down the manager.
+
+        @raise SystemExit: always
+        """
         print 'SHUTTING DOWN'
         reactor.stop()
         raise SystemExit
@@ -97,6 +114,8 @@ class AdminAvatar(base.ManagerAvatar):
         """
         Start the given component.  The component should be sleeping before
         this.
+
+        @type componentState: L{planet.ManagerComponentState}
         """
         self.debug('perspective_componentStart(%r)' % componentState)
         return self.vishnu.componentCreate(componentState)
@@ -104,13 +123,27 @@ class AdminAvatar(base.ManagerAvatar):
     def perspective_componentStop(self, componentState):
         """
         Stop the given component.
+        If the component was sad, we clear its sad state as well,
+        since the stop was explicitly requested by the admin.
+
+        @type componentState: L{planet.ManagerComponentState}
         """
         self.debug('perspective_componentStop(%r)' % componentState)
-        return self.perspective_componentCallRemote(componentState, 'stop')
+        d = self.perspective_componentCallRemote(componentState, 'stop')
+        def clearSadCallback(result):
+            if componentState.get('mood') == planet.moods.sad.value:
+                self.debug('clearing sad mood after stopping component')
+                componentState.set('mood', planet.moods.sleeping.value)
+            return result
+        d.addCallback(clearSadCallback)
+
+        return d
         
     def perspective_componentRestart(self, componentState):
         """
         Restart the given component.
+
+        @type componentState: L{planet.ManagerComponentState}
         """
         self.debug('perspective_componentRestart(%r)' % componentState)
         d = self.perspective_componentStop(componentState)
@@ -123,7 +156,14 @@ class AdminAvatar(base.ManagerAvatar):
         """
         Call a method on the given component on behalf of an admin client.
         
-        @type componentState: L{flumotion.common.planet.ManagerComponentState}
+        @param componentState: state of the component to call the method on
+        @type  componentState: L{planet.ManagerComponentState}
+        @param methodName:     name of the method to call.  Gets proxied to
+                               L{flumotion.component.component.""" \
+                               """BaseComponentMedium}'s remote_(methodName)
+        @type  methodName:     str
+
+        @rtype: L{twisted.internet.defer.Deferred}
         """
         assert isinstance(componentState, planet.ManagerComponentState)
 
@@ -139,7 +179,7 @@ class AdminAvatar(base.ManagerAvatar):
         if methodName == "stop" and componentState.get('mood') == planet.moods.sad.value and not avatar:
             self.debug('asked to stop a sad component without avatar')
             componentState.set('mood', planet.moods.sleeping.value)
-            return
+            return defer.succeed(None)
         
         if not avatar:
             self.warning('No avatar for %s, cannot call remote' %
@@ -151,9 +191,11 @@ class AdminAvatar(base.ManagerAvatar):
         try:
             return avatar.mindCallRemote(methodName, *args, **kwargs)
         except Exception, e:
-            msg = "exception on remote call %s: %s" % (methodName, str(e))
+            msg = "exception on remote call %s: %s" % (methodName,
+                log.getExceptionMessage(e))
             self.warning(msg)
-            raise errors.RemoteMethodError(str(e))
+            raise errors.RemoteMethodError(methodName, 
+                log.getExceptionMessage(e))
 
     def perspective_workerCallRemote(self, workerName, methodName,
                                      *args, **kwargs):
@@ -162,10 +204,12 @@ class AdminAvatar(base.ManagerAvatar):
         This is used so that admin clients can call methods from the interface
         to the worker.
 
-        @type  workerName: string
-        @param workerName: the worker to call.
-        @type  methodName: string
-        @param methodName: the method to call on the worker.
+        @param workerName: the worker to call
+        @type  workerName: str
+        @param methodName: Name of the method to call.  Gets proxied to
+                           L{flumotion.worker.worker.WorkerMedium}'s
+                           remote_(methodName)
+        @type  methodName: str
         """
         
         self.debug('AdminAvatar.workerCallRemote(%r, %r)' % (
@@ -177,14 +221,16 @@ class AdminAvatar(base.ManagerAvatar):
         try:
             return workerAvatar.mindCallRemote(methodName, *args, **kwargs)
         except Exception, e:
-            self.warning("exception on remote call: %s" % str(e))
-            return failure.Failure(errors.RemoteMethodError(str(e)))
+            self.warning("exception on remote call: %s" %
+                log.getExceptionMessage(e))
+            return failure.Failure(errors.RemoteMethodError(methodName,
+                log.getExceptionMessage(e)))
         
     def perspective_getEntryByType(self, componentState, type):
         """
         Get the entry point for a piece of bundled code by the type.
 
-        Returns: a (filename, methodName) tuple, or raises a Failure.
+        Returns: a (filename, methodName) tuple, or raises a Failure
         """
         m = self.vishnu.getComponentMapper(componentState)
         componentName = componentState.get('name')
@@ -214,7 +260,12 @@ class AdminAvatar(base.ManagerAvatar):
         return (filename, entry.function)
 
     def perspective_reloadComponent(self, componentState):
-        """Reload modules in the given component."""
+        """
+        Reload modules in the given component.
+
+        @param componentState: state of the component to reload
+        @type  componentState: L{planet.ManagerComponentState}
+        """
         def _reloaded(result, self, name):
             self.info("reloaded component %s code" % name)
 
@@ -227,7 +278,9 @@ class AdminAvatar(base.ManagerAvatar):
         return d
 
     def perspective_reloadManager(self):
-        """Reload modules in the manager."""
+        """
+        Reload modules in the manager.
+        """
         import sys
         from twisted.python.rebuild import rebuild
         self.info('reloading manager code')
@@ -244,13 +297,14 @@ class AdminAvatar(base.ManagerAvatar):
         """
         Get the configuration of the manager as an XML string.
 
-        @returns: string
+        @rtype: str
         """
         return self.vishnu.getConfiguration()
 
     def perspective_loadConfiguration(self, xml):
         """
-        @type  xml: string
+        Load the given XML configuration into the manager.
+        @type  xml: str
         """
         self.info('loadConfiguration ...')
         self.vishnu.loadConfiguration(None, xml)
@@ -274,29 +328,26 @@ class AdminHeaven(base.ManagerHeaven):
     """
 
     logCategory = "admin-heaven"
-    __implements__ = interfaces.IHeaven
+    implements(interfaces.IHeaven)
     avatarClass = AdminAvatar
 
     def __init__(self, vishnu):
-        """
-        @type vishnu: L{flumotion.manager.manager.Vishnu}
-        @param vishnu: the Vishnu in control of all the heavens
-        """
+        # doc in base class
         base.ManagerHeaven.__init__(self, vishnu)
         #FIXME: don't add a log handler here until we have a good way
         #of filtering client-side again
         #log.addLogHandler(self.logHandler)
-        self.logcache = []
+        self._logcache = []
 
-    def logHandler(self, category, type, message):
-        self.logcache.append((category, type, message))
-        for avatar in self.getAvatars():
-            avatar.sendLog(category, type, message)
+    #def logHandler(self, category, type, message):
+    #    self.logcache.append((category, type, message))
+    #    for avatar in self.getAvatars():
+    #        avatar.sendLog(category, type, message)
 
-    def sendCache(self, avatar):
-        if not avatar.hasRemoteReference():
-            reactor.callLater(0.25, self.sendCache, avatar)
-            return
+    #def sendCache(self, avatar):
+    #    if not avatar.hasRemoteReference():
+    #        reactor.callLater(0.25, self.sendCache, avatar)
+    #        return
         
         # FIXME: do this on request only
         #self.debug('sending logcache to client (%d messages)' % len(self.logcache))
@@ -309,8 +360,10 @@ class AdminHeaven(base.ManagerHeaven):
         """
         Call a remote method on all AdminAvatars in this heaven.
 
-        @type methodName: string
+        @param methodName: Name of the method to call.  Gets proxied to
+                           L{flumotion.admin.admin.AdminModel}'s
+                           remote_(methodName)
+        @type  methodName: str
         """
         for avatar in self.getAvatars():
             avatar.mindCallRemote(methodName, *args, **kwargs)
-  
