@@ -24,7 +24,7 @@ common classes and code to support manager-side objects
 """
 
 from twisted.internet import reactor
-from twisted.spread import pb
+from twisted.spread import pb, flavors
 from twisted.python import failure
 
 from flumotion.common import errors, interfaces, log, common
@@ -32,10 +32,22 @@ from flumotion.common import errors, interfaces, log, common
 class ManagerAvatar(pb.Avatar, log.Loggable):
     """
     I am a base class for manager-side avatars to subclass from.
+
+    @ivar avatarId: the id for this avatar, unique inside the heaven
+    @type avatarId: str
+    @ivar heaven:   the heaven this avatar is part of
+    @type heaven:   L{flumotion.manager.base.ManagerHeaven}
+    @ivar mind:     a remote reference to the client-side Medium
+    @type mind:     L{twisted.spread.pb.RemoteReference}
+    @ivar vishnu:   the vishnu that manages this avatar's heaven
+    @type vishnu:   L{flumotion.manager.manager.Vishnu}
     """
     def __init__(self, heaven, avatarId):
         """
-        @type heaven: L{flumotion.manager.base.ManagerHeaven}
+        @param heaven:   the heaven this avatar is part of
+        @type  heaven:   L{flumotion.manager.base.ManagerHeaven}
+        @param avatarId: id of the avatar to create
+        @type  avatarId: str
         """
         self.heaven = heaven
         self.avatarId = avatarId
@@ -49,7 +61,7 @@ class ManagerAvatar(pb.Avatar, log.Loggable):
         """
         Check if the avatar has a remote reference to the peer.
 
-        @rtype: boolean
+        @rtype: bool
         """
         return self.mind != None
     
@@ -57,6 +69,9 @@ class ManagerAvatar(pb.Avatar, log.Loggable):
     def mindCallRemote(self, name, *args, **kwargs):
         """
         Call the given remote method.
+
+        @param name: name of the remote method
+        @type  name: str
         """
         if not self.hasRemoteReference():
             self.warning(
@@ -88,12 +103,22 @@ class ManagerAvatar(pb.Avatar, log.Loggable):
 
     def _mindCallRemoteErrback(self, f, name):
         if f.check(AttributeError):
-            # FIXME: what if the code raised an actual AttributeError ?
-            # file an issue for twisted
-            # this was done and resolved, can't remember the number now
+            from twisted.spread import flavors
+            if hasattr(flavors, 'NoSuchMethod'):
+                if f.check(flavors.NoSuchMethod):
+                    self.warning("No such remote method '%s'" % name)
+                    raise errors.NoMethodError(name)
+                else:
+                    raise errors.RemoteRunFailure(name,
+                        debug=log.getFailureMessage(f))
+
+            # revision #13473 of Twisted added NoSuchMethod
+            # this is an older version, so we don't know the difference.
             self.warning("No such remote method '%s', or AttributeError "
-                "while executing remote method" % name)
-            return failure.Failure(errors.NoMethodError(name))
+                "while executing remote method (%s)" % (
+                    name, log.getFailureMessage(f)))
+            raise errors.NoMethodError(name,
+                debug=log.getFailureMessage(f))
 
         self.debug("Failure on remote call %s: %r, %s" % (name,
              f, f.getErrorMessage()))
@@ -125,6 +150,8 @@ class ManagerAvatar(pb.Avatar, log.Loggable):
 
         Called through the manager's PB logout trigger calling
         L{flumotion.manager.manager.Dispatcher.removeAvatar}
+
+        @type mind: L{twisted.spread.pb.RemoteReference}
         """
         assert(self.mind == mind)
         self.debug('PB client from %s detached' % self.getClientAddress())
@@ -135,6 +162,9 @@ class ManagerAvatar(pb.Avatar, log.Loggable):
         """
         Get the IPv4 address of the machine the PB client is connecting from,
         as seen from the avatar.
+
+        @returns:  the IPv4 address the client is coming from, or None.
+        @rtype:   str or None
         """
         if self.mind:
             peer = self.mind.broker.transport.getPeer()
@@ -153,14 +183,15 @@ class ManagerAvatar(pb.Avatar, log.Loggable):
         starting with this bundle, in the correct order.
         One of bundleName, fileName, moduleName must be given.
 
-        @type  bundleName: string
         @param bundleName: the name of the bundle for fetching
-        @type  fileName:   string
+        @type  bundleName: str
         @param fileName:   the name of the file requested for fetching
-        @type  moduleName: string
+        @type  fileName:   str
         @param moduleName: the name of the module requested for import
+        @type  moduleName: str
 
-        @rtype: list of (string, string) tuples of (bundleName, md5sum)
+        @returns: list of (bundleName, md5sum) tuples
+        @rtype:   list of tuple of (str, str)
         """
         if bundleName:
             self.debug('asked to get bundle sums for bundle %s' % bundleName)
@@ -169,7 +200,7 @@ class ManagerAvatar(pb.Avatar, log.Loggable):
         elif moduleName:
             self.debug('asked to get bundle sums for module %s' % moduleName)
 
-        basket = self.vishnu.bundlerBasket
+        basket = self.vishnu.getBundlerBasket()
 
         # will raise an error if bundleName not known
         if not bundleName:
@@ -190,7 +221,7 @@ class ManagerAvatar(pb.Avatar, log.Loggable):
                 raise Exception('invalid arguments')
     
         deps = basket.getDependencies(bundleName)
-        self.debug('dependencies of %s: %r' % (bundleName, deps))
+        self.debug('dependencies of %s: %r' % (bundleName, deps[1:]))
         sums = []
         for dep in deps:
             bundler = basket.getBundlerByName(dep)
@@ -208,13 +239,14 @@ class ManagerAvatar(pb.Avatar, log.Loggable):
         Get a list of (bundleName, md5sum) of all dependency bundles,
         starting with this bundle, in the correct order.
 
-        @type  filename: string
         @param filename: the name of the file in a bundle
+        @type  filename: str
 
-        @rtype: list of (string, string) tuples
+        @returns: list of (bundleName, md5sum) tuples
+        @rtype:   list of (str, str) tuples
         """
         self.debug('asked to get bundle sums for file %s' % filename)
-        basket = self.vishnu.bundlerBasket
+        basket = self.vishnu.getBundlerBasket()
         bundleName = basket.getBundlerNameByFile(filename)
         if not bundleName:
             self.warning('Did not find a bundle for file %s' % filename)
@@ -226,12 +258,13 @@ class ManagerAvatar(pb.Avatar, log.Loggable):
         """
         Get the zip files for the given list of bundles.
 
-        @type  bundles: list of string
         @param bundles: the names of the bundles to get
+        @type  bundles: list of str
 
-        @returns: a dictionary of name -> zip data
+        @returns: dictionary of bundleName -> zipdata
+        @rtype:   dict of str -> str
         """
-        basket = self.vishnu.bundlerBasket
+        basket = self.vishnu.getBundlerBasket()
         zips = {}
         for name in bundles:
             bundler = basket.getBundlerByName(name)
@@ -244,25 +277,35 @@ class ManagerAvatar(pb.Avatar, log.Loggable):
 class ManagerHeaven(pb.Root, log.Loggable):
     """
     I am a base class for heavens in the manager.
+
+    @cvar avatarClass: the class object this heaven instantiates avatars from.
+                       To be set in subclass.
+    @ivar avatars:     a dict of avatarId -> Avatar
+    @type avatars:     dict of str -> L{ManagerAvatar}
+    @ivar vishnu:      the Vishnu in control of all the heavens
+    @type vishnu:      L{flumotion.manager.manager.Vishnu}
     """
 
     avatarClass = None
 
     def __init__(self, vishnu):
         """
-        @type vishnu: L{flumotion.manager.manager.Vishnu}
         @param vishnu: the Vishnu in control of all the heavens
+        @type  vishnu: L{flumotion.manager.manager.Vishnu}
         """
         self.vishnu = vishnu
-        self.avatars = {} # name -> avatar
+        self.avatars = {} # avatarId -> avatar
        
     ### ManagerHeaven methods
     def createAvatar(self, avatarId):
         """
-        Create a new administration avatar and manage it.
+        Create a new avatar and manage it.
 
-        @rtype:   L{flumotion.manager.admin.AdminAvatar}
-        @returns: a new avatar for the admin client.
+        @param avatarId: id of the avatar to create
+        @type  avatarId: str
+
+        @returns: a new avatar for the client
+        @rtype:   L{flumotion.manager.base.ManagerAvatar}
         """
         self.debug('creating new Avatar with name %s' % avatarId)
         if self.avatars.has_key(avatarId):
@@ -277,8 +320,8 @@ class ManagerHeaven(pb.Root, log.Loggable):
         """
         Stop managing the given avatar.
 
-        @type avatarId:  string
         @param avatarId: id of the avatar to remove
+        @type  avatarId: str
         """
         self.debug('removing Avatar with id %s' % avatarId)
         del self.avatars[avatarId]
@@ -287,10 +330,11 @@ class ManagerHeaven(pb.Root, log.Loggable):
         """
         Get the avatar with the given id.
 
-        @type  avatarId: string
         @param avatarId: id of the avatar to get
+        @type  avatarId: str
 
-        @rtype: L{ManagerAvatar}
+        @returns: the avatar with the given id
+        @rtype:   L{ManagerAvatar}
         """
         return self.avatars[avatarId]
 
@@ -298,11 +342,11 @@ class ManagerHeaven(pb.Root, log.Loggable):
         """
         Check if a component with that name is registered.
 
-        @type  avatarId: string
         @param avatarId: id of the avatar to check
+        @type  avatarId: str
 
-        @rtype:      boolean
-        @returns:    True if an avatar with that id is registered
+        @returns: True if an avatar with that id is registered
+        @rtype:   bool
         """
         return self.avatars.has_key(avatarId)
 
@@ -310,6 +354,7 @@ class ManagerHeaven(pb.Root, log.Loggable):
         """
         Get all avatars in this heaven.
 
-        @rtype: list of L{ManagerAvatar}
+        @returns: a list of all avatars in this heaven
+        @rtype:   list of L{ManagerAvatar}
         """
         return self.avatars.values()
