@@ -60,7 +60,9 @@ def _startSSL(vishnu, host, port, pemFile):
     if not os.path.split(pemFile)[0]:
         pemFile = os.path.join(configure.configdir, pemFile)
     if not os.path.exists(pemFile):
-        log.error('manager', ".pem file %s does not exist" % pemFile)
+        log.error('manager', ".pem file %s does not exist.\n" \
+            "For more information, see \n" \
+            "http://www.flumotion.net/doc/flumotion/manual/html/chapter-security.html" % pemFile)
     log.debug('manager', 'Using PEM certificate file %s' % pemFile)
     ctxFactory = ServerContextFactory(pemFile)
     
@@ -78,8 +80,14 @@ def _startTCP(vishnu, host, port):
 def _error(message, reason):
     msg = message
     if reason:
-        msg += " - reason: %s" % reason
-    log.error('manager', msg)
+        msg += "\n%s" % reason
+    # since our SystemError is going to be lost in the reactor, we may as well
+    # trap it here
+    # FIXME: maybe we should stop making this raise SystemErrror ?
+    try:
+        log.error('manager', msg)
+    except errors.SystemError:
+        pass
 
 def _initialLoadConfig(vishnu, paths):
     # this is used with a callLater for the initial config loading
@@ -100,7 +108,7 @@ def _initialLoadConfig(vishnu, paths):
             # a re-raise here would be caught by twisted and only shows at
             # debug level 4 because that's where we hooked up twisted logging
             # so print a traceback before stopping the program
-            traceback.print_tb(sys.exc_traceback)
+            traceback.print_tb(sys.exc_info()[2])
             _error("failed to load planet configuration '%s':" % path,
                 "%s: %s" % (e.__class__, str(e)))
 
@@ -153,6 +161,9 @@ def main(args):
                      action="store_true", dest="daemonize",
                      default=False,
                      help="run in background as a daemon")
+    group.add_option('', '--daemonize-to',
+                     action="store", dest="daemonizeTo",
+                     help="what directory to run from when daemonizing")
     parser.add_option_group(group)
     
     log.debug('manager', 'Parsing arguments (%r)' % ', '.join(args))
@@ -176,9 +187,10 @@ def main(args):
     planetFile = args[1]
     try:
         cfg = config.FlumotionConfigXML(planetFile)
-    except IOError:
-        sys.stderr.write("ERROR: Could not read configuration from '%s'.\n" %
+    except IOError, e:
+        sys.stderr.write("ERROR: Could not read configuration from '%s':\n" %
             planetFile)
+        sys.stderr.write("ERROR: %s\n" % e.strerror)
         return -1
     except errors.ConfigError, e:
         sys.stderr.write("ERROR: Could not read configuration from '%s':\n" %
@@ -275,12 +287,18 @@ def main(args):
             _startSSL(vishnu, options.host, options.port, options.certificate)
         elif options.transport == "tcp":
             _startTCP(vishnu, options.host, options.port)
-    except error.CannotListenError, (interface, port, e):
-        # e is a socket.error()
-        message = "Could not listen on port %d: %s" % (port, e.args[1])
+    except error.CannotListenError, e:
+        message = "Could not listen on port %d: %s" % (
+            e.port, e.socketError.args[1])
         raise errors.SystemError, message
 
     log.info('manager', 'Starting manager "%s"' % options.name)
+
+    if options.daemonizeTo and not options.daemonize:
+        sys.stderr.write(
+            'ERROR: --daemonize-to can only be used with -D/--daemonize.\n')
+        return 1
+
     if options.daemonize:
         log.info('manager', 'Daemonizing')
         common.ensureDir(configure.logdir, "log file")
@@ -295,7 +313,11 @@ def main(args):
         log.debug('manager', 'Further logging will be done to %s' % logPath)
 
         # here we daemonize; so we also change our pid
-        common.daemonize(stdout=logPath, stderr=logPath)
+        if not options.daemonizeTo:
+            options.daemonizeTo = '/'
+        common.daemonize(stdout=logPath, stderr=logPath,
+            directory=options.daemonizeTo)
+
         log.info('manager', 'Started daemon')
 
         # from now on I should keep running, whatever happens
