@@ -41,6 +41,7 @@ from flumotion.manager import admin, component, worker, base
 from flumotion.twisted import checkers
 from flumotion.twisted import portal as fportal
 from flumotion.twisted.defer import defer_generator_method
+from flumotion.twisted.compat import implements
 
 from flumotion.common.messages import N_
 T_ = messages.gettexter('flumotion')
@@ -73,7 +74,7 @@ class Dispatcher(log.Loggable):
     the Avatar.
     """
     
-    __implements__ = portal.IRealm
+    implements(portal.IRealm)
 
     logCategory = 'dispatcher'
 
@@ -257,10 +258,17 @@ class Vishnu(log.Loggable):
                 moduleName = defs.getSource()
                 methodName = "createComponent"
             bouncer = job.createComponent(moduleName, methodName)
-            bouncer.setup(configDict)
-            self.setBouncer(bouncer)
-            self.bouncer.debug('started')
-            log.info('manager', "Started manager's bouncer")
+            d = bouncer.setup(configDict)
+            yield d
+            
+            try:
+                d.value()
+                self.setBouncer(bouncer)
+                self.bouncer.debug('started')
+                log.info('manager', "Started manager's bouncer")
+            except errors.ConfigError, e:
+                self.warning('Configuration error in manager bouncer: %s' %
+                    e.args[0])
 
         # now add stuff from the config that did not exist in self.state yet
         self.debug('syncing up planet state with config')
@@ -350,6 +358,7 @@ class Vishnu(log.Loggable):
                 # components
                 for c in ours:
                     components.remove(c)
+    loadConfiguration = defer_generator_method(loadConfiguration)
  
     def _addComponent(self, config, parent):
         """
@@ -399,6 +408,21 @@ class Vishnu(log.Loggable):
         self.dispatcher.registerHeaven(heaven, interface)
         return heaven
     
+    def getBundlerBasket(self):
+        """
+        Return a bundler basket to unbundle from.
+        If the registry files were updated since the last time, the
+        bundlerbasket will be rebuilt.
+
+        @since: 0.2.2
+        @rtype: L{flumotion.common.bundle.BundlerBasket}
+        """
+        if registry.getRegistry().rebuildNeeded():
+            self.info("Registry changed, rebuilding")
+            registry.getRegistry().verify()
+            self.bundlerBasket = registry.getRegistry().makeBundlerBasket()
+        return self.bundlerBasket
+ 
     def setBouncer(self, bouncer):
         """
         @type bouncer: L{flumotion.component.bouncers.bouncer.Bouncer}
@@ -829,3 +853,5 @@ class Vishnu(log.Loggable):
             return self._componentMappers[object]
 
         return None
+
+
