@@ -23,8 +23,6 @@
 manager-side objects to handle worker clients
 """
 
-import socket
-
 from twisted.spread import pb
 from twisted.internet import defer
 
@@ -42,33 +40,37 @@ class PortSet(log.Loggable):
     def __init__(self, workername, ports):
         self.logName = workername
         self.ports = ports
-        self.used = [False] * len(ports)
+        self.used = [0] * len(ports)
 
     def reservePorts(self, numPorts):
         ret = []
         while numPorts > 0:
-            if not False in self.used:
+            if not 0 in self.used:
                 raise errors.ComponentStartError(
                     'could not allocate port on worker %s' % self.logName)
-            i = self.used.index(False)
+            i = self.used.index(0)
             ret.append(self.ports[i])
-            self.used[i] = True
+            self.used[i] = 1
             numPorts -= 1
         return ret
 
     def releasePorts(self, ports):
+        """
+        @param ports: list of ports to release
+        @type  ports: list of int
+        """
         for p in ports:
             try:
                 i = self.ports.index(p)
                 if self.used[i]:
-                    self.used[i] = False
+                    self.used[i] = 0
                 else:
                     self.warning('releasing unallocated port: %d' % p)
             except ValueError:
                 self.warning('releasing unknown port: %d' % p)
 
     def numFree(self):
-        return len(filter(lambda x: not x, self.used))
+        return len([x for x in self.used if x == 0])
     
 class WorkerAvatar(base.ManagerAvatar):
     """
@@ -84,6 +86,7 @@ class WorkerAvatar(base.ManagerAvatar):
         return self.avatarId
 
     def attached(self, mind):
+        # doc in base class
         self.info('worker "%s" logged in' % self.getName())
         base.ManagerAvatar.attached(self, mind)
 
@@ -97,15 +100,28 @@ class WorkerAvatar(base.ManagerAvatar):
     attached = defer_generator_method(attached)
 
     def detached(self, mind):
+        # doc in base class
         self.info('worker "%s" logged out' % self.getName())
         base.ManagerAvatar.detached(self, mind)
         self.heaven.workerDetached(self)
         self.vishnu.workerDetached(self)
     
     def reservePorts(self, numPorts):
+        """
+        Reserve the given number of ports on the worker.
+
+        @param numPorts: how many ports to reserve
+        @type  numPorts: int
+        """
         return self.portset.reservePorts(numPorts)
 
     def releasePorts(self, ports):
+        """
+        Release the given list of ports on the worker.
+    
+        @param ports: list of ports to release
+        @type  ports: list of int
+        """
         self.portset.releasePorts(ports)
 
     def createComponent(self, avatarId, type, config):
@@ -113,9 +129,9 @@ class WorkerAvatar(base.ManagerAvatar):
         Create a component of the given type with the given config.
 
         @param avatarId: avatarId the component should use to log in
-        @type  avatarId: string
+        @type  avatarId: str
         @param type:     type of the component to create
-        @type  type:     string
+        @type  type:     str
         @param config:   a configuration dictionary for the component
         @type  config:   dict
 
@@ -192,7 +208,6 @@ class WorkerHeaven(base.ManagerHeaven):
         @type  workerAvatar: L{WorkerAvatar}
         """
         workerName = workerAvatar.getName()
-        names = self.state.get('names')
         if workerName in self.state.get('names'):
             self.state.remove('names', workerName)
         else:
