@@ -21,12 +21,18 @@
 
 import common
 import testclasses
+import twisted
+import twisted.copyright #T1.3
 
 from twisted.trial import unittest
 
 from twisted.internet import reactor, defer
 from twisted.spread import pb
 from flumotion.twisted import flavors
+from flumotion.twisted.compat import implements
+#T1.3
+def weHaveAnOldTwisted():
+    return twisted.copyright.version < '2.0.0'
 
 class TestStateCacheable(flavors.StateCacheable):
     pass
@@ -38,7 +44,7 @@ pb.setUnjellyableForClass(TestStateCacheable, TestStateRemoteCache)
 class FakeObject: pass
 class FakeListener:
     # listener interface
-    __implements__ = flavors.IStateListener,
+    implements(flavors.IStateListener)
     
     def stateSet(self, state, key, value): pass
     def stateAppend(self, state, key, value): pass
@@ -52,13 +58,13 @@ class TestRoot(testclasses.TestManagerRoot):
         return self.state
 
     def remote_setStateName(self, name):
-        self.state.set('name', name)
+        return self.state.set('name', name)
 
     def remote_bearChild(self, name):
-        self.state.append('children', name)
+        return self.state.append('children', name)
 
     def remote_haveAdopted(self, name):
-        self.state.remove('children', name)
+        return self.state.remove('children', name)
 
 class TestStateSet(unittest.TestCase):
     def setUp(self):
@@ -66,7 +72,7 @@ class TestStateSet(unittest.TestCase):
         self.runServer()
 
     def tearDown(self):
-        unittest.deferredResult(self.stopServer())
+        return self.stopServer()
         
     # helper functions to start PB comms
     def runClient(self):
@@ -74,6 +80,8 @@ class TestStateSet(unittest.TestCase):
         self.cport = reactor.connectTCP("127.0.0.1", self.port, f)
         d = f.getRootObject()
         d.addCallback(self.clientConnected)
+        if weHaveAnOldTwisted():
+            reactor.iterate()
         return d
         #.addCallbacks(self.connected, self.notConnected)
         # self.id = reactor.callLater(10, self.timeOut)
@@ -95,83 +103,88 @@ class TestStateSet(unittest.TestCase):
         self.port = self.sport.getHost().port
 
     def stopServer(self):
-        d = self.sport.stopListening()
-        return d
+        return self.sport.stopListening()
 
     # actual tests
     def testStateSet(self):
-        # start everything
-        unittest.deferredResult(self.runClient())
-        
-        # get the state
-        d = self.perspective.callRemote('getState')
-        state = unittest.deferredResult(d)
+        d = self.runClient()
+        d.addCallback(lambda _: self.perspective.callRemote('getState'))
 
-        self.failUnless(state)
-        self.failUnlessEqual(state.get('name'), 'lois')
-        self.assertRaises(KeyError, state.get, 'dad')
+        def set_state(state):
+            d.state = state
+            self.failUnless(state)
+            self.failUnlessEqual(state.get('name'), 'lois')
+            self.assertRaises(KeyError, state.get, 'dad')
+            return self.perspective.callRemote('setStateName', 'clark')
 
-        # ask server to set the name
-        d = self.perspective.callRemote('setStateName', 'clark')
-        r = unittest.deferredResult(d)
+        def check_name(_):
+            self.failUnlessEqual(d.state.get('name'), 'clark')
 
-        self.failUnlessEqual(state.get('name'), 'clark')
-        unittest.deferredResult(self.stopClient())
+        d.addCallback(set_state)
+        d.addCallback(check_name)
+        d.addCallback(lambda _: self.stopClient())
+        if weHaveAnOldTwisted(): #T1.3
+            return unittest.deferredResult(d)
+        else:
+            return d
 
     def testStateAppendRemove(self):
         # start everything
         d = self.runClient()
-        unittest.deferredResult(d)
-        
-        # get the state
-        d = self.perspective.callRemote('getState')
-        state = unittest.deferredResult(d)
+        d.addCallback(lambda _: self.perspective.callRemote('getState'))
 
-        self.failUnless(state)
-        self.failUnlessEqual(state.get('children'), [])
+        def set_state_and_bear_child(state):
+            d.state = state
+            self.failUnless(state)
+            self.failUnlessEqual(state.get('children'), [])
+            return self.perspective.callRemote('bearChild', 'robin')
 
-        # ask server to make children
-        d = self.perspective.callRemote('bearChild', 'robin')
-        r = unittest.deferredResult(d)
+        def check_first_kid_and_bear_again(_):
+            self.failUnlessEqual(d.state.get('children'), ['robin'])
+            return self.perspective.callRemote('bearChild', 'robin')
 
-        self.failUnlessEqual(state.get('children'), ['robin'])
+        def check_second_kid_and_give_away(_):
+            self.failUnlessEqual(d.state.get('children'), ['robin', 'robin'])
+            return self.perspective.callRemote('haveAdopted', 'robin')
 
-        # lists can have same member more than once
-        d = self.perspective.callRemote('bearChild', 'robin')
-        r = unittest.deferredResult(d)
+        def check_after_adopt_and_bear_again(_):
+            self.failUnlessEqual(d.state.get('children'), ['robin'])
+            return self.perspective.callRemote('bearChild', 'batman')
 
-        self.failUnlessEqual(state.get('children'), ['robin', 'robin'])
+        def check_third_kid_and_stop(_):
+            self.failUnlessEqual(d.state.get('children'), ['robin', 'batman'])
+            return self.stopClient()
 
-        # give one of them away
-        d = self.perspective.callRemote('haveAdopted', 'robin')
-        r = unittest.deferredResult(d)
-
-        self.failUnlessEqual(state.get('children'), ['robin'])
-
-        # add a different one
-        d = self.perspective.callRemote('bearChild', 'batman')
-        r = unittest.deferredResult(d)
-
-        self.failUnlessEqual(state.get('children'), ['robin', 'batman'])
-        unittest.deferredResult(self.stopClient())
+        d.addCallback(set_state_and_bear_child)
+        d.addCallback(check_first_kid_and_bear_again)
+        d.addCallback(check_second_kid_and_give_away)
+        d.addCallback(check_after_adopt_and_bear_again)
+        d.addCallback(check_third_kid_and_stop)
+        if weHaveAnOldTwisted(): #T1.3
+            return unittest.deferredResult(d)
+        else:
+            return d
 
     def testStateWrongListener(self):
         # start everything
         d = self.runClient()
-        unittest.deferredResult(d)
+        d.addCallback(lambda _: self.perspective.callRemote('getState'))
 
-        # get the state
-        d = self.perspective.callRemote('getState')
-        state = unittest.deferredResult(d)
+        def got_state_and_stop(state):
+            self.assertRaises(NotImplementedError, state.addListener, FakeObject())
+            self.assertRaises(NotImplementedError, state.removeListener,
+                              FakeObject())
+            self.assertRaises(KeyError, state.removeListener, FakeListener())
+            return self.stopClient()
 
-        self.assertRaises(NotImplementedError, state.addListener, FakeObject())
-        self.assertRaises(NotImplementedError, state.removeListener,
-            FakeObject())
-        self.assertRaises(KeyError, state.removeListener, FakeListener())
-        unittest.deferredResult(self.stopClient())
+        d.addCallback(got_state_and_stop)
+        if weHaveAnOldTwisted(): #T1.3
+            return unittest.deferredResult(d)
+        else:
+            return d
 
     # listener interface
-    __implements__ = flavors.IStateListener,
+    implements(flavors.IStateListener)
     
     def stateSet(self, state, key, value):
         self.changes.append(('set', state, key, value))
@@ -184,62 +197,61 @@ class TestStateSet(unittest.TestCase):
 
     # listener tests
     def testStateSetListener(self):
-        # start everything
+        # start everything and get the state
         d = self.runClient()
-        unittest.deferredResult(d)
+        d.addCallback(lambda _: self.perspective.callRemote('getState'))
 
-        # get the state
-        d = self.perspective.callRemote('getState')
-        state = unittest.deferredResult(d)
+        # ask server to set the name
+        def add_listener_and_set_name(state):
+            d.state = state # monkeypatch
+            state.addListener(self)
+            return self.perspective.callRemote('setStateName', 'robin')
 
-        state.addListener(self)
+        def check_results(_):
+            c = self.changes.pop()
+            self.failUnlessEqual(c, ('set', d.state, 'name', 'robin'))
+            return self.stopClient()
 
-         # ask server to set the name
-        d = self.perspective.callRemote('setStateName', 'robin')
-        r = unittest.deferredResult(d)
-        c = self.changes.pop()
-        self.failUnlessEqual(c, ('set', state, 'name', 'robin'))
-        unittest.deferredResult(self.stopClient())
+        d.addCallback(add_listener_and_set_name)
+        d.addCallback(check_results)
+        if weHaveAnOldTwisted(): #T1.3
+            return unittest.deferredResult(d)
+        else:
+            return d
 
     def testStateAppendRemoveListener(self):
-        # start everything
+        # start everything and get the state
         d = self.runClient()
-        unittest.deferredResult(d)
-        
-        # get the state
-        d = self.perspective.callRemote('getState')
-        state = unittest.deferredResult(d)
+        d.addCallback(lambda _: self.perspective.callRemote('getState'))
 
-        state.addListener(self)
+        def add_listener_and_bear_child(state):
+            d.state = state # monkeypatch
+            state.addListener(self)
+            return self.perspective.callRemote('bearChild', 'robin')
 
-        # ask server to make children
-        d = self.perspective.callRemote('bearChild', 'robin')
-        r = unittest.deferredResult(d)
+        def check_append_results_and_adopt_kid(_):
+            c = self.changes.pop()
+            self.failUnlessEqual(c, ('append', d.state, 'children', 'robin'))
+            return self.perspective.callRemote('haveAdopted', 'robin')
 
-        c = self.changes.pop()
-        self.failUnlessEqual(c, ('append', state, 'children', 'robin'))
+        def check_remove_results_and_bear_child(_):
+            c = self.changes.pop()
+            self.failUnlessEqual(c, ('remove', d.state, 'children', 'robin'))
+            return self.perspective.callRemote('bearChild', 'batman')
 
-        # lists can have same member more than once
-        d = self.perspective.callRemote('bearChild', 'robin')
-        r = unittest.deferredResult(d)
+        def check_append_results_and_stop(_):
+            c = self.changes.pop()
+            self.failUnlessEqual(c, ('append', d.state, 'children', 'batman'))
+            return self.stopClient()
 
-        c = self.changes.pop()
-        self.failUnlessEqual(c, ('append', state, 'children', 'robin'))
-
-        # give one of them away
-        d = self.perspective.callRemote('haveAdopted', 'robin')
-        r = unittest.deferredResult(d)
-
-        c = self.changes.pop()
-        self.failUnlessEqual(c, ('remove', state, 'children', 'robin'))
-
-        # add a different one
-        d = self.perspective.callRemote('bearChild', 'batman')
-        r = unittest.deferredResult(d)
-
-        c = self.changes.pop()
-        self.failUnlessEqual(c, ('append', state, 'children', 'batman'))
-        unittest.deferredResult(self.stopClient())
+        d.addCallback(add_listener_and_bear_child)
+        d.addCallback(check_append_results_and_adopt_kid)
+        d.addCallback(check_remove_results_and_bear_child)
+        d.addCallback(check_append_results_and_stop)
+        if weHaveAnOldTwisted(): #T1.3
+            return unittest.deferredResult(d)
+        else:
+            return d
 
 class TestState(unittest.TestCase):
     def testStateAddKey(self):
