@@ -24,7 +24,6 @@ Flumotion Twisted-like flavors
 
 Inspired by L{twisted.spread.flavors}
 """
-
 from twisted.internet import defer
 from twisted.python import components
 from twisted.spread import pb
@@ -33,8 +32,10 @@ from twisted.spread import pb
 from flumotion.twisted import compat
 compat.filterWarnings(components, 'ComponentsDeprecationWarning')
 
+from flumotion.twisted.compat import Interface
+
 ### Generice Cacheable/RemoteCache for state objects
-class IStateListener(components.Interface):
+class IStateListener(Interface):
     """
     I am an interface for objects that want to listen to changes on
     cached states.
@@ -210,18 +211,16 @@ class StateRemoteCache(pb.RemoteCache):
                 self._listeners = []
 
     def addListener(self, listener):
-        if not components.implements(listener, IStateListener):
+        if not compat.implementsInterface(listener, IStateListener):
             raise NotImplementedError(
                 '%r instance does not implement IStateListener' % listener)
-
         self._ensureListeners()
         self._listeners.append(listener)
 
     def removeListener(self, listener):
-        if not components.implements(listener, IStateListener):
+        if not compat.implementsInterface(listener, IStateListener):
             raise NotImplementedError(
                 '%r instance does not implement IStateListener' % listener)
-
         self._ensureListeners()
         if listener not in self._listeners:
             raise KeyError(
