@@ -22,9 +22,7 @@
 import errno
 import optparse
 import os
-import signal
 import sys
-import time
 
 from twisted.internet import reactor
 
@@ -33,7 +31,7 @@ from flumotion.common import log, keycards, common, errors
 from flumotion.worker import worker, config
 from flumotion.twisted import credentials
 
-def main(args):
+def _createParser():
     parser = optparse.OptionParser()
     parser.add_option('-d', '--debug',
                       action="store", type="string", dest="debug",
@@ -49,11 +47,12 @@ def main(args):
     group.add_option('-H', '--host',
                      action="store", type="string", dest="host",
                      help="manager to connect to [default localhost]")
-    defaultSSLPort = configure.defaultSSLManagerPort
-    defaultTCPPort = configure.defaultTCPManagerPort
     group.add_option('-P', '--port',
                      action="store", type="int", dest="port",
-                     help="port to listen on [default %d (ssl) or %d (tcp)]" % (defaultSSLPort, defaultTCPPort))
+                     help="manager port to connect to " \
+                        "[default %d (ssl) or %d (tcp)]" % (
+                        configure.defaultSSLManagerPort,
+                        configure.defaultTCPManagerPort))
     group.add_option('-T', '--transport',
                      action="store", type="string", dest="transport",
                      help="transport protocol to use (tcp/ssl) [default ssl]")
@@ -63,6 +62,9 @@ def main(args):
     group.add_option('-D', '--daemonize',
                      action="store_true", dest="daemonize",
                      help="run in background as a daemon")
+    group.add_option('', '--daemonize-to',
+                     action="store", dest="daemonizeTo",
+                     help="what directory to run from when daemonizing")
 
     group.add_option('-u', '--username',
                      action="store", type="string", dest="username",
@@ -78,7 +80,11 @@ def main(args):
                      help="range of feeder ports to use")
 
     parser.add_option_group(group)
+
+    return parser
     
+def main(args):
+    parser = _createParser()
     log.debug('worker', 'Parsing arguments (%r)' % ', '.join(args))
     options, args = parser.parse_args(args)
 
@@ -105,6 +111,10 @@ def main(args):
             raise errors.SystemError(
                 "Could not load configuration from %s: %s" % (
                 workerFile, value))
+        except IOError, e:
+            raise errors.SystemError(
+                "Could not load configuration from %s: %s" % (
+                workerFile, e.strerror))
 
         # now copy over stuff from config that is not set yet
         if not options.name and cfg.name:
@@ -150,9 +160,9 @@ def main(args):
         options.transport = 'ssl'
     if not options.port:
         if options.transport == "tcp":
-            options.port = defaultTCPPort
+            options.port = configure.defaultTCPManagerPort
         elif options.transport == "ssl":
-            options.port = defaultSSLPort
+            options.port = configure.defaultSSLManagerPort
 
     # set a default name if none is given
     if not options.name:
@@ -179,13 +189,21 @@ def main(args):
     if options.debug:
         log.setFluDebug(options.debug)
 
+    if options.daemonizeTo and not options.daemonize:
+        sys.stderr.write(
+            'ERROR: --daemonize-to can only be used with -D/--daemonize.\n')
+        return 1
+
     if options.daemonize:
         common.ensureDir(configure.logdir, "log file")
         common.ensureDir(configure.rundir, "run file")
 
         logPath = os.path.join(configure.logdir, 'worker.%s.log' %
             options.name)
-        common.daemonize(stdout=logPath, stderr=logPath)
+        if not options.daemonizeTo:
+            options.daemonizeTo = '/'
+        common.daemonize(stdout=logPath, stderr=logPath,
+            directory=options.daemonizeTo)
         log.info('worker', 'Started daemon')
 
         # from now on I should keep running until killed, whatever happens
@@ -211,11 +229,11 @@ def main(args):
     # connect the brain to the manager
     if options.transport == "tcp":
         reactor.connectTCP(options.host, options.port,
-            brain.worker_client_factory)
+            brain.workerClientFactory)
     elif options.transport == "ssl":
         from twisted.internet import ssl
         reactor.connectSSL(options.host, options.port,
-            brain.worker_client_factory,
+            brain.workerClientFactory,
             ssl.ClientContextFactory())
 
     log.info('worker',
@@ -243,7 +261,7 @@ def main(args):
 
     log.debug('worker', 'Reactor stopped')
 
-    pids = [kid.getPid() for kid in brain.kindergarten.getKids()]
+    pids = [kid.pid for kid in brain.kindergarten.getKids()]
     
     if pids:
         log.info('worker', 'Waiting for %d jobs to finish' % len(pids))
